@@ -76,7 +76,10 @@
  *       N_("Output tags (also known as metadata)"), NULL},
  *   {NULL}
  *  };
- *  ctx = g_option_context_new ("gst-launch");
+ *  // must initialise the threading system before using any other GLib funtion
+ *  if (!g_thread_supported ())
+ *    g_thread_init (NULL);
+ *  ctx = g_option_context_new ("[ADDITIONAL ARGUMENTS]");
  *  g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
  *  g_option_context_add_group (ctx, gst_init_get_option_group ());
  *  if (!g_option_context_parse (ctx, &amp;argc, &amp;argv, &amp;err)) {
@@ -107,7 +110,12 @@
 #ifdef HAVE_FORK
 #include <sys/wait.h>
 #endif /* HAVE_FORK */
+#ifdef HAVE_SYS_UTSNAME_H
+#include <sys/utsname.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include "gst-i18n-lib.h"
 #include <locale.h>             /* for LC_ALL */
@@ -120,32 +128,38 @@
 #define GST_PLUGIN_SEPARATOR ","
 
 static gboolean gst_initialized = FALSE;
+static gboolean gst_deinitialized = FALSE;
 
 #ifndef GST_DISABLE_REGISTRY
 static GList *plugin_paths = NULL;      /* for delayed processing in post_init */
 #endif
 
-extern gint _gst_trace_on;
+#ifndef GST_DISABLE_GST_DEBUG
+extern const gchar *priv_gst_dump_dot_dir;
+#endif
 
 /* defaults */
 #ifdef HAVE_FORK
-#define DEFAULT_FORK TRUE;
+#define DEFAULT_FORK TRUE
 #else
-#define DEFAULT_FORK FALSE;
+#define DEFAULT_FORK FALSE
 #endif /* HAVE_FORK */
 
-/* set to TRUE when segfaults need to be left as is, FIXME, this variable is
- * global. */
-gboolean _gst_disable_segtrap = FALSE;
+/* set to TRUE when segfaults need to be left as is */
+static gboolean _gst_disable_segtrap = FALSE;
 
 /* control the behaviour of registry rebuild */
 static gboolean _gst_enable_registry_fork = DEFAULT_FORK;
 
 static void load_plugin_func (gpointer data, gpointer user_data);
-static gboolean init_pre (void);
-static gboolean init_post (void);
+static gboolean init_pre (GOptionContext * context, GOptionGroup * group,
+    gpointer data, GError ** error);
+static gboolean init_post (GOptionContext * context, GOptionGroup * group,
+    gpointer data, GError ** error);
+#ifndef GST_DISABLE_OPTION_PARSING
 static gboolean parse_goption_arg (const gchar * s_opt,
     const gchar * arg, gpointer data, GError ** err);
+#endif
 
 static GSList *preload_plugins = NULL;
 
@@ -272,58 +286,84 @@ parse_debug_list (const gchar * list)
  * This function is useful if you want to integrate GStreamer with other
  * libraries that use GOption (see g_option_context_add_group() ).
  *
- * Returns: a pointer to GStreamer's option group. Should be dereferenced
- * after use.
+ * If you use this function, you should make sure you initialise the GLib
+ * threading system as one of the very first things in your program
+ * (see the example at the beginning of this section).
+ *
+ * Returns: a pointer to GStreamer's option group.
  */
 
 GOptionGroup *
 gst_init_get_option_group (void)
 {
+#ifndef GST_DISABLE_OPTION_PARSING
   GOptionGroup *group;
   const static GOptionEntry gst_args[] = {
     {"gst-version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-        parse_goption_arg, N_("Print the GStreamer version"), NULL},
+        (gpointer) parse_goption_arg, N_("Print the GStreamer version"), NULL},
     {"gst-fatal-warnings", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-        parse_goption_arg, N_("Make all warnings fatal"), NULL},
+        (gpointer) parse_goption_arg, N_("Make all warnings fatal"), NULL},
 #ifndef GST_DISABLE_GST_DEBUG
     {"gst-debug-help", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-          parse_goption_arg, N_("Print available debug categories and exit"),
+          (gpointer) parse_goption_arg,
+          N_("Print available debug categories and exit"),
         NULL},
-    {"gst-debug-level", 0, 0, G_OPTION_ARG_CALLBACK, parse_goption_arg,
+    {"gst-debug-level", 0, 0, G_OPTION_ARG_CALLBACK,
+          (gpointer) parse_goption_arg,
           N_("Default debug level from 1 (only error) to 5 (anything) or "
               "0 for no output"),
         N_("LEVEL")},
-    {"gst-debug", 0, 0, G_OPTION_ARG_CALLBACK, parse_goption_arg,
+    {"gst-debug", 0, 0, G_OPTION_ARG_CALLBACK, (gpointer) parse_goption_arg,
           N_("Comma-separated list of category_name:level pairs to set "
               "specific levels for the individual categories. Example: "
               "GST_AUTOPLUG:5,GST_ELEMENT_*:3"),
         N_("LIST")},
     {"gst-debug-no-color", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-        parse_goption_arg, N_("Disable colored debugging output"), NULL},
+          (gpointer) parse_goption_arg, N_("Disable colored debugging output"),
+        NULL},
     {"gst-debug-disable", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-        parse_goption_arg, N_("Disable debugging"), NULL},
+        (gpointer) parse_goption_arg, N_("Disable debugging"), NULL},
 #endif
     {"gst-plugin-spew", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-          parse_goption_arg, N_("Enable verbose plugin loading diagnostics"),
+          (gpointer) parse_goption_arg,
+          N_("Enable verbose plugin loading diagnostics"),
         NULL},
-    {"gst-plugin-path", 0, 0, G_OPTION_ARG_CALLBACK, parse_goption_arg,
+    {"gst-plugin-path", 0, 0, G_OPTION_ARG_CALLBACK,
+          (gpointer) parse_goption_arg,
         N_("Colon-separated paths containing plugins"), N_("PATHS")},
-    {"gst-plugin-load", 0, 0, G_OPTION_ARG_CALLBACK, parse_goption_arg,
+    {"gst-plugin-load", 0, 0, G_OPTION_ARG_CALLBACK,
+          (gpointer) parse_goption_arg,
           N_("Comma-separated list of plugins to preload in addition to the "
               "list stored in environment variable GST_PLUGIN_PATH"),
         N_("PLUGINS")},
     {"gst-disable-segtrap", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
-          parse_goption_arg,
+          (gpointer) parse_goption_arg,
           N_("Disable trapping of segmentation faults during plugin loading"),
         NULL},
     {"gst-disable-registry-fork", 0, G_OPTION_FLAG_NO_ARG,
           G_OPTION_ARG_CALLBACK,
-          parse_goption_arg,
+          (gpointer) parse_goption_arg,
           N_("Disable the use of fork() while scanning the registry"),
         NULL},
     {NULL}
   };
 
+  /* The GLib threading system must be initialised before calling any other
+   * GLib function according to the documentation; if the application hasn't
+   * called gst_init() yet or initialised the threading system otherwise, we
+   * better issue a warning here (since chances are high that the application
+   * has already called other GLib functions such as g_option_context_new() */
+  if (!g_thread_supported ()) {
+    g_warning ("The GStreamer function gst_init_get_option_group() was\n"
+        "\tcalled, but the GLib threading system has not been initialised\n"
+        "\tyet, something that must happen before any other GLib function\n"
+        "\tis called. The application needs to be fixed so that it calls\n"
+        "\t   if (!g_thread_supported ()) g_thread_init(NULL);\n"
+        "\tas very first thing in its main() function. Please file a bug\n"
+        "\tagainst this application.");
+    g_thread_init (NULL);
+  }
+
   group = g_option_group_new ("gst", _("GStreamer Options"),
       _("Show GStreamer Options"), NULL, NULL);
   g_option_group_set_parse_hooks (group, (GOptionParseFunc) init_pre,
@@ -333,6 +373,9 @@ gst_init_get_option_group (void)
   g_option_group_set_translation_domain (group, GETTEXT_PACKAGE);
 
   return group;
+#else
+  return NULL;
+#endif
 }
 
 /**
@@ -348,28 +391,40 @@ gst_init_get_option_group (void)
  * for some reason.  If you want your program to fail fatally,
  * use gst_init() instead.
  *
+ * This function should be called before calling any other GLib functions. If
+ * this is not an option, your program must initialise the GLib thread system
+ * using g_thread_init() before any other GLib functions are called.
+ *
  * Returns: %TRUE if GStreamer could be initialized.
  */
 gboolean
 gst_init_check (int *argc, char **argv[], GError ** err)
 {
+#ifndef GST_DISABLE_OPTION_PARSING
   GOptionGroup *group;
   GOptionContext *ctx;
+#endif
   gboolean res;
 
-  GST_INFO ("initializing GStreamer");
+  if (!g_thread_supported ())
+    g_thread_init (NULL);
 
   if (gst_initialized) {
     GST_DEBUG ("already initialized gst");
     return TRUE;
   }
-
+#ifndef GST_DISABLE_OPTION_PARSING
   ctx = g_option_context_new ("- GStreamer initialization");
   g_option_context_set_ignore_unknown_options (ctx, TRUE);
   group = gst_init_get_option_group ();
   g_option_context_add_group (ctx, group);
   res = g_option_context_parse (ctx, argc, argv, err);
   g_option_context_free (ctx);
+#else
+  init_pre (NULL, NULL, NULL, NULL);
+  init_post (NULL, NULL, NULL, NULL);
+  res = TRUE;
+#endif
 
   gst_initialized = res;
 
@@ -390,6 +445,10 @@ gst_init_check (int *argc, char **argv[], GError ** err)
  * Initializes the GStreamer library, setting up internal path lists,
  * registering built-in elements, and loading standard plugins.
  *
+ * This function should be called before calling any other GLib functions. If
+ * this is not an option, your program must initialise the GLib thread system
+ * using g_thread_init() before any other GLib functions are called.
+ *
  * <note><para>
  * This function will terminate your program if it was unable to initialize
  * GStreamer for some reason.  If you want your program to fall back,
@@ -425,11 +484,13 @@ add_path_func (gpointer data, gpointer user_data)
 }
 #endif
 
+#ifndef GST_DISABLE_OPTION_PARSING
 static void
 prepare_for_load_plugin_func (gpointer data, gpointer user_data)
 {
   preload_plugins = g_slist_prepend (preload_plugins, g_strdup (data));
 }
+#endif
 
 static void
 load_plugin_func (gpointer data, gpointer user_data)
@@ -449,16 +510,15 @@ load_plugin_func (gpointer data, gpointer user_data)
   } else {
     if (err) {
       /* Report error to user, and free error */
-      GST_ERROR ("Failed to load plugin: %s\n", err->message);
+      GST_ERROR ("Failed to load plugin: %s", err->message);
       g_error_free (err);
     } else {
       GST_WARNING ("Failed to load plugin: \"%s\"", filename);
     }
   }
-
-  g_free (data);
 }
 
+#ifndef GST_DISABLE_OPTION_PARSING
 static void
 split_and_iterate (const gchar * stringlist, gchar * separator, GFunc iterator,
     gpointer user_data)
@@ -483,11 +543,18 @@ split_and_iterate (const gchar * stringlist, gchar * separator, GFunc iterator,
     g_strfreev (strings);
   }
 }
+#endif
 
 /* we have no fail cases yet, but maybe in the future */
 static gboolean
-init_pre (void)
+init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
+    GError ** error)
 {
+  if (gst_initialized) {
+    GST_DEBUG ("already initialized");
+    return TRUE;
+  }
+
   /* GStreamer was built against a GLib >= 2.8 and is therefore not doing
    * the refcount hack. Check that it isn't being run against an older GLib */
   if (glib_major_version < 2 ||
@@ -500,12 +567,8 @@ init_pre (void)
 
   g_type_init ();
 
-  if (g_thread_supported ()) {
-    /* somebody already initialized threading */
-  } else {
-    g_thread_init (NULL);
-  }
   /* we need threading to be enabled right here */
+  g_assert (g_thread_supported ());
   _gst_debug_init ();
 
 #ifdef ENABLE_NLS
@@ -525,12 +588,27 @@ init_pre (void)
       parse_debug_list (debug_list);
     }
   }
+
+  priv_gst_dump_dot_dir = g_getenv ("GST_DEBUG_DUMP_DOT_DIR");
 #endif
   /* This is the earliest we can make stuff show up in the logs.
    * So give some useful info about GStreamer here */
   GST_INFO ("Initializing GStreamer Core Library version %s", VERSION);
   GST_INFO ("Using library installed in %s", LIBDIR);
 
+  /* Print some basic system details if possible (OS/architecture) */
+#ifdef HAVE_SYS_UTSNAME_H
+  {
+    struct utsname sys_details;
+
+    if (uname (&sys_details) == 0) {
+      GST_INFO ("%s %s %s %s %s", sys_details.sysname,
+          sys_details.nodename, sys_details.release, sys_details.version,
+          sys_details.machine);
+    }
+  }
+#endif
+
   return TRUE;
 }
 
@@ -548,54 +626,44 @@ gst_register_core_elements (GstPlugin * plugin)
   return TRUE;
 }
 
-static GstPluginDesc plugin_desc = {
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "staticelements",
-  "core elements linked into the GStreamer library",
-  gst_register_core_elements,
-  VERSION,
-  GST_LICENSE,
-  PACKAGE,
-  GST_PACKAGE_NAME,
-  GST_PACKAGE_ORIGIN,
-
-  GST_PADDING_INIT
-};
-
 #ifndef GST_DISABLE_REGISTRY
 
+typedef enum
+{
+  REGISTRY_SCAN_AND_UPDATE_FAILURE = 0,
+  REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED,
+  REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED
+} GstRegistryScanAndUpdateResult;
+
 /*
  * scan_and_update_registry:
  * @default_registry: the #GstRegistry
  * @registry_file: registry filename
  * @write_changes: write registry if it has changed?
  *
- * Scans for registry changes and evntualy updates the registry cache. 
+ * Scans for registry changes and eventually updates the registry cache. 
  *
- * Return: %TRUE if the registry could be updated
+ * Return: %REGISTRY_SCAN_AND_UPDATE_FAILURE if the registry could not scanned
+ *         or updated, %REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED if the
+ *         registry is clean and %REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED if
+ *         it has been updated and the cache needs to be re-read.
  */
-static gboolean
+static GstRegistryScanAndUpdateResult
 scan_and_update_registry (GstRegistry * default_registry,
-    const gchar * registry_file, gboolean write_changes)
+    const gchar * registry_file, gboolean write_changes, GError ** error)
 {
   const gchar *plugin_path;
   gboolean changed = FALSE;
   GList *l;
 
-  GST_DEBUG ("reading registry cache: %s", registry_file);
-  gst_registry_xml_read_cache (default_registry, registry_file);
-
   /* scan paths specified via --gst-plugin-path */
   GST_DEBUG ("scanning paths added via --gst-plugin-path");
   for (l = plugin_paths; l != NULL; l = l->next) {
     GST_INFO ("Scanning plugin path: \"%s\"", (gchar *) l->data);
     /* CHECKME: add changed |= here as well? */
     gst_registry_scan_path (default_registry, (gchar *) l->data);
-    g_free (l->data);
   }
-  g_list_free (plugin_paths);
-  plugin_paths = NULL;
+  /* keep plugin_paths around in case a re-scan is forced later on */
 
   /* GST_PLUGIN_PATH specifies a list of directories to scan for
    * additional plugins.  These take precedence over the system plugins */
@@ -627,10 +695,12 @@ scan_and_update_registry (GstRegistry * default_registry,
      * system-installed ones */
     home_plugins = g_build_filename (g_get_home_dir (),
         ".gstreamer-" GST_MAJORMINOR, "plugins", NULL);
+    GST_DEBUG ("scanning home plugins %s", home_plugins);
     changed |= gst_registry_scan_path (default_registry, home_plugins);
     g_free (home_plugins);
 
     /* add the main (installed) library path */
+    GST_DEBUG ("scanning main plugins %s", PLUGINDIR);
     changed |= gst_registry_scan_path (default_registry, PLUGINDIR);
   } else {
     gchar **list;
@@ -644,34 +714,48 @@ scan_and_update_registry (GstRegistry * default_registry,
     g_strfreev (list);
   }
 
+  /* Remove cached plugins so stale info is cleared. */
+  changed |= _priv_gst_registry_remove_cache_plugins (default_registry);
+
   if (!changed) {
-    GST_DEBUG ("registry cache has not changed");
-    return TRUE;
+    GST_INFO ("Registry cache has not changed");
+    return REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED;
   }
 
   if (!write_changes) {
-    GST_DEBUG ("not trying to write registry cache changes to file");
-    return TRUE;
+    GST_INFO ("Registry cached changed, but writing is disabled. Not writing.");
+    return REGISTRY_SCAN_AND_UPDATE_FAILURE;
   }
 
-  GST_DEBUG ("writing registry cache");
+  GST_INFO ("Registry cache changed. Writing new registry cache");
+#ifdef USE_BINARY_REGISTRY
+  if (!gst_registry_binary_write_cache (default_registry, registry_file)) {
+#else
   if (!gst_registry_xml_write_cache (default_registry, registry_file)) {
-    g_warning ("Problem writing registry cache to %s: %s", registry_file,
-        g_strerror (errno));
-    return FALSE;
+#endif
+    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+        _("Error writing registry cache to %s: %s"),
+        registry_file, g_strerror (errno));
+    return REGISTRY_SCAN_AND_UPDATE_FAILURE;
   }
 
-  GST_DEBUG ("registry cache written successfully");
-  return TRUE;
+  GST_INFO ("Registry cache written successfully");
+  return REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED;
 }
 
 static gboolean
 ensure_current_registry_nonforking (GstRegistry * default_registry,
-    const gchar * registry_file)
+    const gchar * registry_file, GError ** error)
 {
   /* fork() not available */
-  GST_DEBUG ("updating registry cache");
-  scan_and_update_registry (default_registry, registry_file, TRUE);
+  GST_INFO ("reading registry cache: %s", registry_file);
+#ifdef USE_BINARY_REGISTRY
+  gst_registry_binary_read_cache (default_registry, registry_file);
+#else
+  gst_registry_xml_read_cache (default_registry, registry_file);
+#endif
+  GST_DEBUG ("Updating registry cache in-process");
+  scan_and_update_registry (default_registry, registry_file, TRUE, error);
   return TRUE;
 }
 
@@ -679,68 +763,110 @@ ensure_current_registry_nonforking (GstRegistry * default_registry,
  * TRUE immediatly */
 static gboolean
 ensure_current_registry_forking (GstRegistry * default_registry,
-    const gchar * registry_file)
+    const gchar * registry_file, GError ** error)
 {
 #ifdef HAVE_FORK
   pid_t pid;
+  int pfd[2];
+  int ret;
 
   /* We fork here, and let the child read and possibly rebuild the registry.
    * After that, the parent will re-read the freshly generated registry. */
-  GST_DEBUG ("forking");
+  GST_DEBUG ("forking to update registry");
+
+  if (pipe (pfd) == -1) {
+    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+        _("Error re-scanning registry %s: %s"),
+        ", could not create pipes. Error", g_strerror (errno));
+    return FALSE;
+  }
+
+  GST_INFO ("reading registry cache: %s", registry_file);
+#ifdef USE_BINARY_REGISTRY
+  gst_registry_binary_read_cache (default_registry, registry_file);
+#else
+  gst_registry_xml_read_cache (default_registry, registry_file);
+#endif
+
   pid = fork ();
   if (pid == -1) {
     GST_ERROR ("Failed to fork()");
+    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+        _("Error re-scanning registry %s: %s"),
+        ", failed to fork. Error", g_strerror (errno));
     return FALSE;
   }
 
   if (pid == 0) {
-    gboolean res;
+    gint result_code;
+
+    /* this is the child. Close the read pipe */
+    (void) close (pfd[0]);
 
-    /* this is the child */
     GST_DEBUG ("child reading registry cache");
-    res = scan_and_update_registry (default_registry, registry_file, TRUE);
-    _gst_registry_remove_cache_plugins (default_registry);
+    result_code =
+        scan_and_update_registry (default_registry, registry_file, TRUE, NULL);
 
     /* need to use _exit, so that any exit handlers registered don't
      * bring down the main program */
-    GST_DEBUG ("child exiting: %s", (res) ? "SUCCESS" : "FAILURE");
+    GST_DEBUG ("child exiting: %d", result_code);
 
     /* make valgrind happy (yes, you can call it insane) */
     g_free ((char *) registry_file);
 
-    _exit ((res) ? EXIT_SUCCESS : EXIT_FAILURE);
+    /* write a result byte to the pipe */
+    do {
+      ret = write (pfd[1], &result_code, sizeof (result_code));
+    } while (ret == -1 && errno == EINTR);
+    /* if ret == -1 now, we could not write to pipe, probably 
+     * means parent has exited before us */
+    (void) close (pfd[1]);
+
+    _exit (0);
   } else {
-    /* parent */
-    int status;
-    pid_t ret;
+    gint result_code;
+
+    /* parent. Close write pipe */
+    (void) close (pfd[1]);
+
+    /* Wait for result from the pipe */
+    GST_DEBUG ("Waiting for data from child");
+    do {
+      ret = read (pfd[0], &result_code, sizeof (result_code));
+    } while (ret == -1 && errno == EINTR);
 
-    GST_DEBUG ("parent waiting on child");
-    ret = waitpid (pid, &status, 0);
-    GST_DEBUG ("parent done waiting on child");
     if (ret == -1) {
-      GST_ERROR ("error during waitpid: %s", g_strerror (errno));
+      g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+          _("Error re-scanning registry %s: %s"),
+          ", read returned error", g_strerror (errno));
+      close (pfd[0]);
+      return FALSE;
+    }
+    (void) close (pfd[0]);
+
+    /* Wait to ensure the child is reaped, but ignore the result */
+    GST_DEBUG ("parent waiting on child");
+    waitpid (pid, NULL, 0);
+    GST_DEBUG ("parent done waiting on child");
+
+    if (ret == 0) {
+      GST_ERROR ("child did not exit normally, terminated by signal");
+      g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
+          _("Error re-scanning registry %s"), ", child terminated by signal");
       return FALSE;
     }
 
-    if (!WIFEXITED (status)) {
-      if (WIFSIGNALED (status)) {
-        GST_ERROR ("child did not exit normally, terminated by signal %d",
-            WTERMSIG (status));
-      } else {
-        GST_ERROR ("child did not exit normally, status: %d", status);
-      }
-      return FALSE;
-    }
-
-    GST_DEBUG ("child exited normally with return value %d",
-        WEXITSTATUS (status));
-
-    if (WEXITSTATUS (status) == EXIT_SUCCESS) {
-      GST_DEBUG ("parent reading registry cache");
+    if (result_code == REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED) {
+      GST_DEBUG ("Child succeeded. Parent reading registry cache");
+      _priv_gst_registry_remove_cache_plugins (default_registry);
+#ifdef USE_BINARY_REGISTRY
+      gst_registry_binary_read_cache (default_registry, registry_file);
+#else
       gst_registry_xml_read_cache (default_registry, registry_file);
-    } else {
-      GST_DEBUG ("parent re-scanning registry");
-      scan_and_update_registry (default_registry, registry_file, FALSE);
+#endif
+    } else if (result_code == REGISTRY_SCAN_AND_UPDATE_FAILURE) {
+      GST_DEBUG ("Child failed. Parent re-scanning registry, ignoring errors.");
+      scan_and_update_registry (default_registry, registry_file, FALSE, NULL);
     }
   }
 #endif /* HAVE_FORK */
@@ -748,7 +874,7 @@ ensure_current_registry_forking (GstRegistry * default_registry,
 }
 
 static gboolean
-ensure_current_registry (void)
+ensure_current_registry (GError ** error)
 {
   char *registry_file;
   GstRegistry *default_registry;
@@ -758,8 +884,13 @@ ensure_current_registry (void)
   default_registry = gst_registry_get_default ();
   registry_file = g_strdup (g_getenv ("GST_REGISTRY"));
   if (registry_file == NULL) {
+#ifdef USE_BINARY_REGISTRY
+    registry_file = g_build_filename (g_get_home_dir (),
+        ".gstreamer-" GST_MAJORMINOR, "registry." HOST_CPU ".bin", NULL);
+#else
     registry_file = g_build_filename (g_get_home_dir (),
         ".gstreamer-" GST_MAJORMINOR, "registry." HOST_CPU ".xml", NULL);
+#endif
   }
 
   /* first see if forking is enabled */
@@ -777,10 +908,12 @@ ensure_current_registry (void)
   /* now check registry with or without forking */
   if (do_fork) {
     GST_DEBUG ("forking for registry rebuild");
-    ret = ensure_current_registry_forking (default_registry, registry_file);
+    ret = ensure_current_registry_forking (default_registry, registry_file,
+        error);
   } else {
     GST_DEBUG ("requested not to fork for registry rebuild");
-    ret = ensure_current_registry_nonforking (default_registry, registry_file);
+    ret = ensure_current_registry_nonforking (default_registry, registry_file,
+        error);
   }
 
   g_free (registry_file);
@@ -801,7 +934,8 @@ ensure_current_registry (void)
  *   we might and then it's nice to be able to return that
  */
 static gboolean
-init_post (void)
+init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
+    GError ** error)
 {
   GLogLevelFlags llf;
 
@@ -809,21 +943,26 @@ init_post (void)
   GstTrace *gst_trace;
 #endif /* GST_DISABLE_TRACE */
 
+  if (gst_initialized) {
+    GST_DEBUG ("already initialized");
+    return TRUE;
+  }
+
   llf = G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL;
   g_log_set_handler (g_log_domain_gstreamer, llf, debug_log_handler, NULL);
 
   _priv_gst_quarks_initialize ();
   _gst_format_initialize ();
   _gst_query_initialize ();
-  gst_object_get_type ();
-  gst_pad_get_type ();
-  gst_element_factory_get_type ();
-  gst_element_get_type ();
-  gst_type_find_factory_get_type ();
-  gst_bin_get_type ();
+  g_type_class_ref (gst_object_get_type ());
+  g_type_class_ref (gst_pad_get_type ());
+  g_type_class_ref (gst_element_factory_get_type ());
+  g_type_class_ref (gst_element_get_type ());
+  g_type_class_ref (gst_type_find_factory_get_type ());
+  g_type_class_ref (gst_bin_get_type ());
 
 #ifndef GST_DISABLE_INDEX
-  gst_index_factory_get_type ();
+  g_type_class_ref (gst_index_factory_get_type ());
 #endif /* GST_DISABLE_INDEX */
 #ifndef GST_DISABLE_URI
   gst_uri_handler_get_type ();
@@ -831,17 +970,21 @@ init_post (void)
 
   gst_structure_get_type ();
   _gst_value_initialize ();
+  g_type_class_ref (gst_param_spec_fraction_get_type ());
   gst_caps_get_type ();
   _gst_event_initialize ();
   _gst_buffer_initialize ();
   _gst_message_initialize ();
   _gst_tag_initialize ();
 
-  /* register core plugins */
-  _gst_plugin_register_static (&plugin_desc);
-
   _gst_plugin_initialize ();
 
+  /* register core plugins */
+  gst_plugin_register_static (GST_VERSION_MAJOR, GST_VERSION_MINOR,
+      "staticelements", "core elements linked into the GStreamer library",
+      gst_register_core_elements, VERSION, GST_LICENSE, PACKAGE,
+      GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
+
   /*
    * Any errors happening below this point are non-fatal, we therefore mark
    * gstreamer as being initialized, since it is the case from a plugin point of
@@ -854,16 +997,14 @@ init_post (void)
   gst_initialized = TRUE;
 
 #ifndef GST_DISABLE_REGISTRY
-  if (!ensure_current_registry ())
+  if (!ensure_current_registry (error))
     return FALSE;
 #endif /* GST_DISABLE_REGISTRY */
 
-  /* if we need to preload plugins */
-  if (preload_plugins) {
-    g_slist_foreach (preload_plugins, load_plugin_func, NULL);
-    g_slist_free (preload_plugins);
-    preload_plugins = NULL;
-  }
+  /* if we need to preload plugins, do so now */
+  g_slist_foreach (preload_plugins, load_plugin_func, NULL);
+  /* keep preload_plugins around in case a re-scan is forced later on */
+
 #ifndef GST_DISABLE_TRACE
   _gst_trace_on = 0;
   if (_gst_trace_on) {
@@ -888,13 +1029,15 @@ sort_by_category_name (gconstpointer a, gconstpointer b)
   return strcmp (gst_debug_category_get_name ((GstDebugCategory *) a),
       gst_debug_category_get_name ((GstDebugCategory *) b));
 }
+
 static void
 gst_debug_help (void)
 {
   GSList *list, *walk;
   GList *list2, *g;
 
-  if (!init_post ())
+  /* Need to ensure the registry is loaded to get debug categories */
+  if (!init_post (NULL, NULL, NULL, NULL))
     exit (1);
 
   list2 = gst_registry_plugin_filter (gst_registry_get_default (),
@@ -941,6 +1084,7 @@ gst_debug_help (void)
 }
 #endif
 
+#ifndef GST_DISABLE_OPTION_PARSING
 static gboolean
 parse_one_option (gint opt, const gchar * arg, GError ** err)
 {
@@ -1043,6 +1187,7 @@ parse_goption_arg (const gchar * opt,
 
   return parse_one_option (val, arg, err);
 }
+#endif
 
 extern GstRegistry *_gst_registry_default;
 
@@ -1065,18 +1210,39 @@ gst_deinit (void)
 
   GST_INFO ("deinitializing GStreamer");
 
-  if (!gst_initialized) {
+  if (gst_deinitialized) {
     GST_DEBUG ("already deinitialized");
     return;
   }
 
+  g_slist_foreach (preload_plugins, (GFunc) g_free, NULL);
+  g_slist_free (preload_plugins);
+  preload_plugins = NULL;
+
+#ifndef GST_DISABLE_REGISTRY
+  g_list_foreach (plugin_paths, (GFunc) g_free, NULL);
+  g_list_free (plugin_paths);
+  plugin_paths = NULL;
+#endif
+
   clock = gst_system_clock_obtain ();
   gst_object_unref (clock);
   gst_object_unref (clock);
 
-  _gst_registry_cleanup ();
+  _priv_gst_registry_cleanup ();
 
-  gst_initialized = FALSE;
+  g_type_class_unref (g_type_class_peek (gst_object_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_element_factory_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_element_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_type_find_factory_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_bin_get_type ()));
+#ifndef GST_DISABLE_INDEX
+  g_type_class_unref (g_type_class_peek (gst_index_factory_get_type ()));
+#endif /* GST_DISABLE_INDEX */
+  g_type_class_unref (g_type_class_peek (gst_param_spec_fraction_get_type ()));
+
+  gst_deinitialized = TRUE;
   GST_INFO ("deinitialized GStreamer");
 }
 
@@ -1203,3 +1369,55 @@ gst_registry_fork_set_enabled (gboolean enabled)
   _gst_enable_registry_fork = enabled;
 #endif /* HAVE_FORK */
 }
+
+
+/**
+ * gst_update_registry:
+ *
+ * Forces GStreamer to re-scan its plugin paths and update the default
+ * plugin registry.
+ *
+ * Applications will almost never need to call this function, it is only
+ * useful if the application knows new plugins have been installed (or old
+ * ones removed) since the start of the application (or, to be precise, the
+ * first call to gst_init()) and the application wants to make use of any
+ * newly-installed plugins without restarting the application.
+ *
+ * Applications should assume that the registry update is neither atomic nor
+ * thread-safe and should therefore not have any dynamic pipelines running
+ * (including the playbin and decodebin elements) and should also not create
+ * any elements or access the GStreamer registry while the update is in
+ * progress.
+ *
+ * Note that this function may block for a significant amount of time.
+ *
+ * Returns: %TRUE if the registry has been updated successfully (does not
+ *          imply that there were changes), otherwise %FALSE.
+ *
+ * Since: 0.10.12
+ */
+gboolean
+gst_update_registry (void)
+{
+  gboolean res = FALSE;
+
+#ifndef GST_DISABLE_REGISTRY
+  GError *err = NULL;
+
+  res = ensure_current_registry (&err);
+  if (err) {
+    GST_WARNING ("registry update failed: %s", err->message);
+    g_error_free (err);
+  } else {
+    GST_LOG ("registry update succeeded");
+  }
+
+  if (preload_plugins) {
+    g_slist_foreach (preload_plugins, load_plugin_func, NULL);
+  }
+#else
+  GST_WARNING ("registry update failed: %s", "registry disabled");
+#endif /* GST_DISABLE_REGISTRY */
+
+  return res;
+}
