@@ -26,8 +26,8 @@
  * They are composed of an array of #GstStructure.
  *
  * Caps are exposed on #GstPadTemplate to describe all possible types a
- * given pad can handle. They are also stored in the registry along with
- * a description of the element.
+ * given pad can handle. They are also stored in the #GstRegistry along with
+ * a description of the #GstElement.
  *
  * Caps are exposed on the element pads using the gst_pad_get_caps() pad
  * function. This function describes the possible types that the pad can
@@ -45,7 +45,7 @@
  *  GstCaps *caps;
  *  caps = gst_caps_new_simple ("video/x-raw-yuv",
  *       "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
- *       "framerate", G_TYPE_DOUBLE, 25.0,
+ *       "framerate", GST_TYPE_FRACTION, 25, 1,
  *       "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1,
  *       "width", G_TYPE_INT, 320,
  *       "height", G_TYPE_INT, 240,
@@ -60,7 +60,7 @@
  * Various methods exist to work with the media types such as subtracting
  * or intersecting.
  *
- * Last reviewed on 2005-11-23 (0.9.5)
+ * Last reviewed on 2007-02-13 (0.10.10)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -99,6 +99,9 @@
 #define FREE_CAPS(caps) g_free (caps)
 #endif
 
+/* lock to protect multiple invocations of static caps to caps conversion */
+G_LOCK_DEFINE_STATIC (static_caps_lock);
+
 static void gst_caps_transform_to_string (const GValue * src_value,
     GValue * dest_value);
 static gboolean gst_caps_from_string_inplace (GstCaps * caps,
@@ -302,6 +305,10 @@ _gst_caps_free (GstCaps * caps)
 #ifdef USE_POISONING
   memset (caps, 0xff, sizeof (GstCaps));
 #endif
+
+#ifdef DEBUG_REFCOUNT
+  GST_CAT_LOG (GST_CAT_CAPS, "freeing caps %p", caps);
+#endif
   FREE_CAPS (caps);
 }
 
@@ -427,18 +434,49 @@ gst_static_caps_get (GstStaticCaps * static_caps)
 
   caps = (GstCaps *) static_caps;
 
-  if (caps->type == 0) {
-    if (G_UNLIKELY (static_caps->string == NULL))
+  /* refcount is 0 when we need to convert */
+  if (G_UNLIKELY (g_atomic_int_get (&caps->refcount) == 0)) {
+    const char *string;
+    GstCaps temp;
+
+    G_LOCK (static_caps_lock);
+    /* check if other thread already updated */
+    if (G_UNLIKELY (g_atomic_int_get (&caps->refcount) > 0))
+      goto done;
+
+    string = static_caps->string;
+
+    if (G_UNLIKELY (string == NULL))
       goto no_string;
 
-    caps->type = GST_TYPE_CAPS;
-    /* initialize the caps to a refcount of 1 so the caps can be writable... */
-    gst_atomic_int_set (&caps->refcount, 1);
-    caps->structs = g_ptr_array_new ();
+    GST_CAT_LOG (GST_CAT_CAPS, "creating %p", static_caps);
+
+    /* we construct the caps on the stack, then copy over the struct into our
+     * real caps, refcount last. We do this because we must leave the refcount
+     * of the result caps to 0 so that other threads don't run away with the
+     * caps while we are constructing it. */
+    temp.type = GST_TYPE_CAPS;
+    temp.flags = 0;
+    temp.structs = g_ptr_array_new ();
+
+    /* initialize the caps to a refcount of 1 so the caps can be writable for
+     * the next statement */
+    gst_atomic_int_set (&temp.refcount, 1);
 
     /* convert to string */
-    if (G_UNLIKELY (!gst_caps_from_string_inplace (caps, static_caps->string)))
-      g_critical ("Could not convert static caps \"%s\"", static_caps->string);
+    if (G_UNLIKELY (!gst_caps_from_string_inplace (&temp, string)))
+      g_critical ("Could not convert static caps \"%s\"", string);
+
+    /* now copy stuff over to the real caps. */
+    caps->type = temp.type;
+    caps->flags = temp.flags;
+    caps->structs = temp.structs;
+    /* and bump the refcount so other threads can now read */
+    gst_atomic_int_set (&caps->refcount, 1);
+
+    GST_CAT_LOG (GST_CAT_CAPS, "created %p", static_caps);
+  done:
+    G_UNLOCK (static_caps_lock);
   }
   /* ref the caps, makes it not writable */
   gst_caps_ref (caps);
@@ -448,7 +486,8 @@ gst_static_caps_get (GstStaticCaps * static_caps)
   /* ERRORS */
 no_string:
   {
-    g_warning ("static caps string is NULL");
+    G_UNLOCK (static_caps_lock);
+    g_warning ("static caps %p string is NULL", static_caps);
     return NULL;
   }
 }
@@ -856,7 +895,7 @@ gst_caps_set_simple_valist (GstCaps * caps, char *field, va_list varargs)
   GstStructure *structure;
 
   g_return_if_fail (GST_IS_CAPS (caps));
-  g_return_if_fail (caps->structs->len != 1);
+  g_return_if_fail (caps->structs->len == 1);
   g_return_if_fail (IS_WRITABLE (caps));
 
   structure = gst_caps_get_structure (caps, 0);
@@ -990,11 +1029,11 @@ gst_caps_is_always_compatible (const GstCaps * caps1, const GstCaps * caps2)
  * @subset: a #GstCaps
  * @superset: a potentially greater #GstCaps
  *
- * Checks if all caps represented by @subset are also represented by @superset
+ * Checks if all caps represented by @subset are also represented by @superset.
  * <note>This function does not work reliably if optional properties for caps
  * are included on one caps and omitted on the other.</note>
  *
- * Returns: TRUE if @subset is a subset of @superset
+ * Returns: %TRUE if @subset is a subset of @superset
  */
 gboolean
 gst_caps_is_subset (const GstCaps * subset, const GstCaps * superset)
@@ -1175,9 +1214,15 @@ gst_caps_intersect (const GstCaps * caps1, const GstCaps * caps2)
   g_return_val_if_fail (GST_IS_CAPS (caps1), NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps2), NULL);
 
-  if (gst_caps_is_empty (caps1) || gst_caps_is_empty (caps2)) {
+  /* caps are exactly the same pointers, just copy one caps */
+  if (caps1 == caps2)
+    return gst_caps_copy (caps1);
+
+  /* empty caps on either side, return empty */
+  if (gst_caps_is_empty (caps1) || gst_caps_is_empty (caps2))
     return gst_caps_new_empty ();
-  }
+
+  /* one of the caps is any, just copy the other caps */
   if (gst_caps_is_any (caps1))
     return gst_caps_copy (caps2);
   if (gst_caps_is_any (caps2))
@@ -1728,7 +1773,7 @@ gst_caps_replace (GstCaps ** caps, GstCaps * newcaps)
 gchar *
 gst_caps_to_string (const GstCaps * caps)
 {
-  guint i;
+  guint i, slen;
   GString *s;
 
   /* NOTE:  This function is potentially called by the debug system,
@@ -1747,20 +1792,28 @@ gst_caps_to_string (const GstCaps * caps)
     return g_strdup ("EMPTY");
   }
 
-  s = g_string_new ("");
+  /* estimate a rough string length to avoid unnecessary reallocs in GString */
+  slen = 0;
   for (i = 0; i < caps->structs->len; i++) {
-    GstStructure *structure;
-    char *sstr;
-
-    if (i > 0)
-      g_string_append (s, "; ");
-
-    structure = gst_caps_get_structure (caps, i);
-    sstr = gst_structure_to_string (structure);
-    g_string_append (s, sstr);
-    g_free (sstr);
+    slen += STRUCTURE_ESTIMATED_STRING_LEN (gst_caps_get_structure (caps, i));
   }
 
+  s = g_string_sized_new (slen);
+  for (i = 0; i < caps->structs->len; i++) {
+    GstStructure *structure;
+
+    if (i > 0) {
+      /* ';' is now added by gst_structure_to_string */
+      g_string_append_c (s, ' ');
+    }
+
+    structure = gst_caps_get_structure (caps, i);
+    priv_gst_structure_append_to_gstring (structure, s);
+  }
+  if (s->len && s->str[s->len - 1] == ';') {
+    /* remove latest ';' */
+    s->str[--s->len] = '\0';
+  }
   return g_string_free (s, FALSE);
 }
 
@@ -1785,22 +1838,20 @@ gst_caps_from_string_inplace (GstCaps * caps, const gchar * string)
   }
   gst_caps_append_structure (caps, structure);
 
-  while (*s == ';') {
-    s++;
+  do {
+
     while (g_ascii_isspace (*s))
       s++;
+    if (*s == '\0') {
+      break;
+    }
     structure = gst_structure_from_string (s, &s);
     if (structure == NULL) {
       return FALSE;
     }
     gst_caps_append_structure (caps, structure);
-    while (g_ascii_isspace (*s))
-      s++;
-  }
 
-  if (*s != 0) {
-    return FALSE;
-  }
+  } while (TRUE);
 
   return TRUE;
 }
