@@ -86,7 +86,7 @@
  * slaved to another #GstClock with the gst_clock_set_master(). The clock will
  * then automatically be synchronized to this master clock by repeadedly
  * sampling the master clock and the slave clock and recalibrating the slave
- * clock with gst_clock_set_calibration(). This feature is mostly usefull for
+ * clock with gst_clock_set_calibration(). This feature is mostly useful for
  * plugins that have an internal clock but must operate with another clock
  * selected by the #GstPipeline.  They can track the offset and rate difference
  * of their internal clock relative to the master clock by using the
@@ -281,6 +281,7 @@ gst_clock_new_periodic_id (GstClock * clock, GstClockTime start_time,
   g_return_val_if_fail (GST_IS_CLOCK (clock), NULL);
   g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (start_time), NULL);
   g_return_val_if_fail (interval != 0, NULL);
+  g_return_val_if_fail (GST_CLOCK_TIME_IS_VALID (interval), NULL);
 
   return gst_clock_entry_new (clock,
       start_time, interval, GST_CLOCK_ENTRY_PERIODIC);
@@ -412,9 +413,8 @@ gst_clock_id_wait (GstClockID id, GstClockTimeDiff * jitter)
   GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
       "done waiting entry %p, res: %d", id, res);
 
-  if (entry->type == GST_CLOCK_ENTRY_PERIODIC) {
-    entry->time += entry->interval;
-  }
+  if (entry->type == GST_CLOCK_ENTRY_PERIODIC)
+    entry->time = requested + entry->interval;
 
   if (clock->stats)
     gst_clock_update_stats (clock);
@@ -733,23 +733,47 @@ gst_clock_get_resolution (GstClock * clock)
  * @clock: a #GstClock to use
  * @internal: a clock time
  *
- * Converts the given @internal clock time to the real time, adjusting for the
+ * Converts the given @internal clock time to the external time, adjusting for the
  * rate and reference time set with gst_clock_set_calibration() and making sure
  * that the returned time is increasing. This function should be called with the
  * clock's OBJECT_LOCK held and is mainly used by clock subclasses.
  *
- * Returns: the converted time of the clock.
+ * This function is te reverse of gst_clock_unadjust_unlocked().
  *
- * MT safe.
+ * Returns: the converted time of the clock.
  */
 GstClockTime
 gst_clock_adjust_unlocked (GstClock * clock, GstClockTime internal)
 {
-  GstClockTime ret;
+  GstClockTime ret, cinternal, cexternal, cnum, cdenom;
 
-  ret = gst_util_uint64_scale (internal - clock->internal_calibration,
-      clock->rate_numerator, clock->rate_denominator);
-  ret += clock->external_calibration;
+  /* get calibration values for readability */
+  cinternal = clock->internal_calibration;
+  cexternal = clock->external_calibration;
+  cnum = clock->rate_numerator;
+  cdenom = clock->rate_denominator;
+
+  /* avoid divide by 0 */
+  if (cdenom == 0)
+    cnum = cdenom = 1;
+
+  /* The formula is (internal - cinternal) * cnum / cdenom + cexternal
+   *
+   * Since we do math on unsigned 64-bit ints we have to special case for
+   * interal < cinternal to get the sign right. this case is not very common,
+   * though.
+   */
+  if (G_LIKELY (internal >= cinternal)) {
+    ret = gst_util_uint64_scale (internal - cinternal, cnum, cdenom);
+    ret += cexternal;
+  } else {
+    ret = gst_util_uint64_scale (cinternal - internal, cnum, cdenom);
+    /* clamp to 0 */
+    if (cexternal > ret)
+      ret = cexternal - ret;
+    else
+      ret = 0;
+  }
 
   /* make sure the time is increasing */
   clock->last_time = MAX (ret, clock->last_time);
@@ -757,6 +781,51 @@ gst_clock_adjust_unlocked (GstClock * clock, GstClockTime internal)
   return clock->last_time;
 }
 
+/**
+ * gst_clock_unadjust_unlocked
+ * @clock: a #GstClock to use
+ * @external: an external clock time
+ *
+ * Converts the given @external clock time to the internal time of @clock,
+ * using the rate and reference time set with gst_clock_set_calibration().
+ * This function should be called with the clock's OBJECT_LOCK held and
+ * is mainly used by clock subclasses.
+ *
+ * This function is te reverse of gst_clock_adjust_unlocked().
+ *
+ * Returns: the internal time of the clock corresponding to @external.
+ *
+ * Since: 0.10.13
+ */
+GstClockTime
+gst_clock_unadjust_unlocked (GstClock * clock, GstClockTime external)
+{
+  GstClockTime ret, cinternal, cexternal, cnum, cdenom;
+
+  /* get calibration values for readability */
+  cinternal = clock->internal_calibration;
+  cexternal = clock->external_calibration;
+  cnum = clock->rate_numerator;
+  cdenom = clock->rate_denominator;
+
+  /* avoid divide by 0 */
+  if (cnum == 0)
+    cnum = cdenom = 1;
+
+  /* The formula is (external - cexternal) * cdenom / cnum + cinternal */
+  if (external >= cexternal) {
+    ret = gst_util_uint64_scale (external - cexternal, cdenom, cnum);
+    ret += cinternal;
+  } else {
+    ret = gst_util_uint64_scale (cexternal - external, cdenom, cnum);
+    if (cinternal > ret)
+      ret = cinternal - ret;
+    else
+      ret = 0;
+  }
+  return ret;
+}
+
 /**
  * gst_clock_get_internal_time
  * @clock: a #GstClock to query
@@ -874,6 +943,12 @@ gst_clock_set_calibration (GstClock * clock, GstClockTime internal, GstClockTime
   g_return_if_fail (internal <= gst_clock_get_internal_time (clock));
 
   GST_OBJECT_LOCK (clock);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+      "internal %" GST_TIME_FORMAT " external %" GST_TIME_FORMAT " %"
+      G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT " = %f", GST_TIME_ARGS (internal),
+      GST_TIME_ARGS (external), rate_num, rate_denom,
+      gst_guint64_to_gdouble (rate_num / rate_denom));
+
   clock->internal_calibration = internal;
   clock->external_calibration = external;
   clock->rate_numerator = rate_num;
@@ -974,7 +1049,8 @@ gst_clock_set_master (GstClock * clock, GstClock * master)
   if (master && !GST_OBJECT_FLAG_IS_SET (clock, GST_CLOCK_FLAG_CAN_SET_MASTER))
     goto not_supported;
 
-  GST_DEBUG_OBJECT (clock, "slaving to master clock %p", master);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+      "slaving %p to master clock %p", clock, master);
   master_p = &clock->master;
   gst_object_replace ((GstObject **) master_p, (GstObject *) master);
   GST_OBJECT_UNLOCK (clock);
@@ -1002,7 +1078,8 @@ gst_clock_set_master (GstClock * clock, GstClock * master)
   /* ERRORS */
 not_supported:
   {
-    GST_DEBUG_OBJECT (clock, "cannot be slaved to a master clock");
+    GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+        "cannot be slaved to a master clock");
     GST_OBJECT_UNLOCK (clock);
     return FALSE;
   }
@@ -1057,9 +1134,10 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   n = clock->filling ? clock->time_index : clock->window_size;
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("doing regression on:");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "doing regression on:");
   for (i = j = 0; i < n; i++, j += 4)
-    GST_DEBUG ("  %" G_GUINT64_FORMAT "  %" G_GUINT64_FORMAT, x[j], y[j]);
+    GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+        "  %" G_GUINT64_FORMAT "  %" G_GUINT64_FORMAT, x[j], y[j]);
 #endif
 
   xmin = ymin = G_MAXUINT64;
@@ -1069,8 +1147,10 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   }
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("min x: %" G_GUINT64_FORMAT, xmin);
-  GST_DEBUG ("min y: %" G_GUINT64_FORMAT, ymin);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "min x: %" G_GUINT64_FORMAT,
+      xmin);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "min y: %" G_GUINT64_FORMAT,
+      ymin);
 #endif
 
   newx = clock->times + 1;
@@ -1083,9 +1163,10 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   }
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("reduced numbers:");
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "reduced numbers:");
   for (i = j = 0; i < n; i++, j += 4)
-    GST_DEBUG ("  %" G_GUINT64_FORMAT "  %" G_GUINT64_FORMAT, newx[j], newy[j]);
+    GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock,
+        "  %" G_GUINT64_FORMAT "  %" G_GUINT64_FORMAT, newx[j], newy[j]);
 #endif
 
   /* have to do this precisely otherwise the results are pretty much useless.
@@ -1101,8 +1182,10 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   ybar /= n;
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("  xbar  = %" G_GUINT64_FORMAT, xbar);
-  GST_DEBUG ("  ybar  = %" G_GUINT64_FORMAT, ybar);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  xbar  = %" G_GUINT64_FORMAT,
+      xbar);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  ybar  = %" G_GUINT64_FORMAT,
+      ybar);
 #endif
 
   /* multiplying directly would give quantities on the order of 1e20 -> 60 bits;
@@ -1134,17 +1217,20 @@ do_linear_regression (GstClock * clock, GstClockTime * m_num,
   *r_squared = ((double) sxy * (double) sxy) / ((double) sxx * (double) syy);
 
 #ifdef DEBUGGING_ENABLED
-  GST_DEBUG ("  m      = %g", ((double) *m_num) / *m_denom);
-  GST_DEBUG ("  b      = %" G_GUINT64_FORMAT, *b);
-  GST_DEBUG ("  xbase  = %" G_GUINT64_FORMAT, *xbase);
-  GST_DEBUG ("  r2     = %g", *r_squared);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  m      = %g",
+      ((double) *m_num) / *m_denom);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  b      = %" G_GUINT64_FORMAT,
+      *b);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  xbase  = %" G_GUINT64_FORMAT,
+      *xbase);
+  GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "  r2     = %g", *r_squared);
 #endif
 
   return TRUE;
 
 invalid:
   {
-    GST_DEBUG_OBJECT (clock, "sxx == 0, regression failed");
+    GST_CAT_DEBUG_OBJECT (GST_CAT_CLOCK, clock, "sxx == 0, regression failed");
     return FALSE;
   }
 }
