@@ -37,7 +37,7 @@
  * The event should be unreffed with gst_event_unref() if it has not been sent.
  *
  * Events that have been received can be parsed with their respective 
- * gst_event_parse_*() functions.
+ * gst_event_parse_*() functions. It is valid to pass %NULL for unwanted details.
  *
  * Events are passed between elements in parallel to the data stream. Some events
  * are serialized with buffers, others are not. Some events only travel downstream,
@@ -89,12 +89,14 @@ static void gst_event_class_init (gpointer g_class, gpointer class_data);
 static void gst_event_finalize (GstEvent * event);
 static GstEvent *_gst_event_copy (GstEvent * event);
 
+static GstMiniObjectClass *parent_class = NULL;
+
 void
 _gst_event_initialize (void)
 {
-  gst_event_get_type ();
-  gst_seek_flags_get_type ();
-  gst_seek_type_get_type ();
+  g_type_class_ref (gst_event_get_type ());
+  g_type_class_ref (gst_seek_flags_get_type ());
+  g_type_class_ref (gst_seek_type_get_type ());
 }
 
 typedef struct
@@ -219,6 +221,8 @@ gst_event_class_init (gpointer g_class, gpointer class_data)
 {
   GstEventClass *event_class = GST_EVENT_CLASS (g_class);
 
+  parent_class = g_type_class_peek_parent (g_class);
+
   event_class->mini_object_class.copy =
       (GstMiniObjectCopyFunction) _gst_event_copy;
   event_class->mini_object_class.finalize =
@@ -252,6 +256,8 @@ gst_event_finalize (GstEvent * event)
     gst_structure_set_parent_refcount (event->structure, NULL);
     gst_structure_free (event->structure);
   }
+
+  GST_MINI_OBJECT_CLASS (parent_class)->finalize (GST_MINI_OBJECT (event));
 }
 
 static GstEvent *
@@ -351,14 +357,14 @@ gst_event_get_structure (GstEvent * event)
 /**
  * gst_event_new_flush_start:
  *
- * Allocate a new flush start event. The flush start event can be send
+ * Allocate a new flush start event. The flush start event can be sent
  * upstream and downstream and travels out-of-bounds with the dataflow.
  *
  * It marks pads as being flushing and will make them return
  * #GST_FLOW_WRONG_STATE when used for data flow with gst_pad_push(),
  * gst_pad_chain(), gst_pad_alloc_buffer(), gst_pad_get_range() and
  * gst_pad_pull_range(). Any event (except a #GST_EVENT_FLUSH_STOP) received
- * on a flushing pad will return %FALSE immediatly.
+ * on a flushing pad will return %FALSE immediately.
  *
  * Elements should unlock any blocking functions and exit their streaming
  * functions as fast as possible when this event is received.
@@ -377,9 +383,9 @@ gst_event_new_flush_start (void)
 /**
  * gst_event_new_flush_stop:
  *
- * Allocate a new flush stop event. The flush start event can be send
+ * Allocate a new flush stop event. The flush stop event can be sent
  * upstream and downstream and travels out-of-bounds with the dataflow.
- * It is typically send after sending a FLUSH_START event to make the
+ * It is typically sent after sending a FLUSH_START event to make the
  * pads accept data again.
  *
  * Elements can process this event synchronized with the dataflow since
