@@ -23,7 +23,7 @@
 
 /**
  * SECTION:gstinfo
- * @short_description: Debugging and logging facillities
+ * @short_description: Debugging and logging facilities
  * @see_also: #GstConfig, #Gst for command line parameters
  * and environment variables that affect the debugging output.
  *
@@ -77,7 +77,7 @@
  * subsystem was compiled out, #GST_DISABLE_GST_DEBUG is defined in
  * &lt;gst/gst.h&gt;,
  * so you can check that before doing your trick.
- * Disabling the debugging subsystem will give you a slight (read: unnoticable)
+ * Disabling the debugging subsystem will give you a slight (read: unnoticeable)
  * speed increase and will reduce the size of your compiled code. The GStreamer
  * library itself becomes around 10% smaller.
  *
@@ -104,6 +104,7 @@
 #  include <process.h>          /* getpid on win32 */
 #endif
 #include <string.h>             /* G_VA_COPY */
+
 #include "gst_private.h"
 #include "gstutils.h"
 #include "gstsegment.h"
@@ -114,8 +115,12 @@
 
 /* underscore is to prevent conflict with GST_CAT_DEBUG define */
 GST_DEBUG_CATEGORY_STATIC (_GST_CAT_DEBUG);
-/* time of initialization, so we get useful debugging output times */
-static GstClockTime start_time;
+
+/* time of initialization, so we get useful debugging output times
+ * FIXME: we use this in gstdebugutils.c, what about a function + macro to
+ * get the running time: GST_DEBUG_RUNNING_TIME
+ */
+GstClockTime _priv_gst_info_start_time;
 
 #if 0
 #if defined __sgi__
@@ -232,7 +237,7 @@ GstDebugCategory *GST_CAT_QOS = NULL;
 
 /* FIXME: export this? */
 gboolean
-__gst_in_valgrind (void)
+_priv_gst_in_valgrind (void)
 {
   static enum
   {
@@ -272,14 +277,11 @@ __gst_in_valgrind (void)
 void
 _gst_debug_init (void)
 {
-  GTimeVal current;
-
   gst_atomic_int_set (&__default_level, GST_LEVEL_DEFAULT);
   gst_atomic_int_set (&__use_color, 1);
 
   /* get time we started for debugging messages */
-  g_get_current_time (&current);
-  start_time = GST_TIMEVAL_TO_TIME (current);
+  _priv_gst_info_start_time = gst_util_get_timestamp ();
 
 #ifdef HAVE_PRINTF_EXTENSION
   register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension_ptr,
@@ -356,7 +358,7 @@ _gst_debug_init (void)
 
 
   /* print out the valgrind message if we're in valgrind */
-  __gst_in_valgrind ();
+  _priv_gst_in_valgrind ();
 }
 
 /* we can't do this further above, because we initialize the GST_CAT_DEFAULT struct */
@@ -511,6 +513,8 @@ gst_debug_print_object (gpointer ptr)
   return g_strdup_printf ("%p", ptr);
 }
 
+#ifdef HAVE_PRINTF_EXTENSION
+
 static gchar *
 gst_debug_print_segment (gpointer ptr)
 {
@@ -553,6 +557,8 @@ gst_debug_print_segment (gpointer ptr)
   }
 }
 
+#endif /* HAVE_PRINTF_EXTENSION */
+
 /**
  * gst_debug_construct_term_color:
  * @colorinfo: the color info
@@ -567,15 +573,14 @@ gchar *
 gst_debug_construct_term_color (guint colorinfo)
 {
   GString *color;
-  gchar *ret;
 
   color = g_string_new ("\033[00");
 
   if (colorinfo & GST_DEBUG_BOLD) {
-    g_string_append (color, ";01");
+    g_string_append_len (color, ";01", 3);
   }
   if (colorinfo & GST_DEBUG_UNDERLINE) {
-    g_string_append (color, ";04");
+    g_string_append_len (color, ";04", 3);
   }
   if (colorinfo & GST_DEBUG_FG_MASK) {
     g_string_append_printf (color, ";3%1d", colorinfo & GST_DEBUG_FG_MASK);
@@ -584,11 +589,9 @@ gst_debug_construct_term_color (guint colorinfo)
     g_string_append_printf (color, ";4%1d",
         (colorinfo & GST_DEBUG_BG_MASK) >> 4);
   }
-  g_string_append (color, "m");
+  g_string_append_c (color, 'm');
 
-  ret = color->str;
-  g_string_free (color, FALSE);
-  return ret;
+  return g_string_free (color, FALSE);
 }
 
 /**
@@ -620,7 +623,6 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
   gchar pidcolor[10];
   const gchar *levelcolor;
   gint pid;
-  GTimeVal now;
   GstClockTime elapsed;
   gboolean free_color = TRUE;
   gboolean free_obj = TRUE;
@@ -660,8 +662,8 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
     free_obj = FALSE;
   }
 
-  g_get_current_time (&now);
-  elapsed = GST_TIMEVAL_TO_TIME (now) - start_time;
+  elapsed = GST_CLOCK_DIFF (_priv_gst_info_start_time,
+      gst_util_get_timestamp ());
 
   /*
      g_printerr ("%s (%p - %" GST_TIME_FORMAT ") %s%20s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
@@ -688,7 +690,7 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
  * gst_debug_level_get_name:
  * @level: the level to get the name for
  *
- * Get the string trepresentation of a debugging level
+ * Get the string representation of a debugging level
  *
  * Returns: the name
  */
@@ -751,7 +753,7 @@ gst_debug_add_log_function (GstLogFunction func, gpointer data)
 static gint
 gst_debug_compare_log_function_by_func (gconstpointer entry, gconstpointer func)
 {
-  gpointer entryfunc = ((LogFuncEntry *) entry)->func;
+  gpointer entryfunc = (gpointer) (((LogFuncEntry *) entry)->func);
 
   return (entryfunc < func) ? -1 : (entryfunc > func) ? 1 : 0;
 }
@@ -793,7 +795,7 @@ gst_debug_remove_with_compare_func (GCompareFunc func, gpointer data)
  * gst_debug_remove_log_function:
  * @func: the log function to remove
  *
- * Removes all registrered instances of the given logging functions.
+ * Removes all registered instances of the given logging functions.
  *
  * Returns: How many instances of the function were removed
  */
@@ -806,7 +808,7 @@ gst_debug_remove_log_function (GstLogFunction func)
 
   removals =
       gst_debug_remove_with_compare_func
-      (gst_debug_compare_log_function_by_func, func);
+      (gst_debug_compare_log_function_by_func, (gpointer) func);
   GST_DEBUG ("removed log function %p %d times from log function list", func,
       removals);
 
@@ -817,7 +819,7 @@ gst_debug_remove_log_function (GstLogFunction func)
  * gst_debug_remove_log_function_by_data:
  * @data: user data of the log function to remove
  *
- * Removes all registrered instances of log functions with the given user data.
+ * Removes all registered instances of log functions with the given user data.
  *
  * Returns: How many instances of the function were removed
  */
@@ -1027,7 +1029,8 @@ gst_debug_unset_threshold_for_name (const gchar * name)
 }
 
 GstDebugCategory *
-_gst_debug_category_new (gchar * name, guint color, gchar * description)
+_gst_debug_category_new (const gchar * name, guint color,
+    const gchar * description)
 {
   GstDebugCategory *cat;
 
@@ -1122,7 +1125,7 @@ gst_debug_category_reset_threshold (GstDebugCategory * category)
  * gst_debug_category_get_threshold:
  * @category: a #GstDebugCategory to get threshold of.
  *
- * Returns the threshold of a #GstCategory.
+ * Returns the threshold of a #GstDebugCategory.
  *
  * Returns: the #GstDebugLevel that is used as threshold.
  */
@@ -1208,11 +1211,10 @@ _gst_debug_nameof_funcptr (GstDebugFuncPtr ptr)
 /* This function MUST NOT return NULL */
      const gchar *_gst_debug_nameof_funcptr (GstDebugFuncPtr func)
 {
-  gpointer ptr = (gpointer) func;
   gchar *ptrname;
 
 #ifdef HAVE_DLADDR
-  Dl_info dlinfo;
+  Dl_info dl_info;
 #endif
 
   if (G_UNLIKELY (func == NULL))
@@ -1220,7 +1222,7 @@ _gst_debug_nameof_funcptr (GstDebugFuncPtr ptr)
 
   g_static_mutex_lock (&__dbg_functions_mutex);
   if (G_LIKELY (__gst_function_pointers)) {
-    ptrname = g_hash_table_lookup (__gst_function_pointers, ptr);
+    ptrname = g_hash_table_lookup (__gst_function_pointers, (gpointer) func);
     g_static_mutex_unlock (&__dbg_functions_mutex);
     if (G_LIKELY (ptrname))
       return ptrname;
@@ -1230,23 +1232,23 @@ _gst_debug_nameof_funcptr (GstDebugFuncPtr ptr)
   /* we need to create an entry in the hash table for this one so we don't leak
    * the name */
 #ifdef HAVE_DLADDR
-  if (dladdr (ptr, &dlinfo) && dlinfo.dli_sname) {
-    gchar *name = g_strdup (dlinfo.dli_sname);
+  if (dladdr ((gpointer) func, &dl_info) && dl_info.dli_sname) {
+    gchar *name = g_strdup (dl_info.dli_sname);
 
-    _gst_debug_register_funcptr (ptr, name);
+    _gst_debug_register_funcptr (func, name);
     return name;
   } else
 #endif
   {
-    gchar *name = g_strdup_printf ("%p", ptr);
+    gchar *name = g_strdup_printf ("%p", (gpointer) func);
 
-    _gst_debug_register_funcptr (ptr, name);
+    _gst_debug_register_funcptr (func, name);
     return name;
   }
 }
 
 void
-_gst_debug_register_funcptr (GstDebugFuncPtr func, gchar * ptrname)
+_gst_debug_register_funcptr (GstDebugFuncPtr func, const gchar * ptrname)
 {
   gpointer ptr = (gpointer) func;
 
@@ -1255,11 +1257,13 @@ _gst_debug_register_funcptr (GstDebugFuncPtr func, gchar * ptrname)
   if (!__gst_function_pointers)
     __gst_function_pointers = g_hash_table_new (g_direct_hash, g_direct_equal);
   if (!g_hash_table_lookup (__gst_function_pointers, ptr))
-    g_hash_table_insert (__gst_function_pointers, ptr, ptrname);
+    g_hash_table_insert (__gst_function_pointers, ptr, (gpointer) ptrname);
 
   g_static_mutex_unlock (&__dbg_functions_mutex);
 }
 
+/*** PRINTF EXTENSIONS ********************************************************/
+
 #ifdef HAVE_PRINTF_EXTENSION
 static int
 _gst_info_printf_extension_ptr (FILE * stream, const struct printf_info *info,
@@ -1323,7 +1327,7 @@ gst_debug_remove_log_function_by_data (gpointer data)
 }
 
 gboolean
-__gst_in_valgrind (void)
+_priv_gst_in_valgrind (void)
 {
   return FALSE;
 }
@@ -1370,7 +1374,7 @@ __cyg_profile_func_exit (void *this_fn, void *call_site)
 /**
  * gst_debug_print_stack_trace:
  *
- * If GST_ENABLE_FUNC_INSTRUMENTATION is defined a stacktracke is available for
+ * If GST_ENABLE_FUNC_INSTRUMENTATION is defined a stacktrace is available for
  * gstreamer code, which can be printed with this function.
  */
 void
@@ -1397,4 +1401,4 @@ gst_debug_print_stack_trace (void)
   /* nothing because it's compiled out */
 }
 
-#endif /* GST_ENABLE_FUNC_INTSTRUMENTATION */
+#endif /* GST_ENABLE_FUNC_INSTRUMENTATION */
