@@ -85,7 +85,7 @@ gst_uri_handler_base_init (gpointer g_class)
 {
   static gboolean initialized = FALSE;
 
-  if (!initialized) {
+  if (G_UNLIKELY (!initialized)) {
 
     /**
      * GstURIHandler::new-uri:
@@ -285,7 +285,7 @@ unescape_string (const gchar * escaped_string, const gchar * illegal_characters)
   }
 
   *out = '\0';
-  g_assert ((size_t) (out - result) <= strlen (escaped_string));
+  g_assert ((gsize) (out - result) <= strlen (escaped_string));
   return result;
 
 }
@@ -369,7 +369,7 @@ gst_uri_get_protocol (const gchar * uri)
 
   colon = strstr (uri, "://");
 
-  return g_strndup (uri, colon - uri);
+  return g_strdown (g_strndup (uri, colon - uri));
 }
 
 /**
@@ -377,7 +377,9 @@ gst_uri_get_protocol (const gchar * uri)
  * @uri: an URI string
  * @protocol: a protocol string (e.g. "http")
  *
- * Returns TRUE if the protocol of a given valid URI matches @protocol.
+ * Checks if the protocol of a given valid URI matches @protocol.
+ *
+ * Returns: %TRUE if the protocol matches.
  *
  * Since: 0.10.4
  */
@@ -395,35 +397,50 @@ gst_uri_has_protocol (const gchar * uri, const gchar * protocol)
   if (colon == NULL)
     return FALSE;
 
-  return (strncmp (uri, protocol, (size_t) (colon - uri)) == 0);
+  return (strncmp (uri, protocol, (gsize) (colon - uri)) == 0);
 }
 
 /**
  * gst_uri_get_location:
  * @uri: A URI string
  *
- * Extracts the location out of a given valid URI. So the protocol and "://"
- * are stripped from the URI. The returned string must be freed using
+ * Extracts the location out of a given valid URI, ie. the protocol and "://"
+ * are stripped from the URI, which means that the location returned includes
+ * the hostname if one is specified. The returned string must be freed using
  * g_free().
  *
- * Returns: The location for this URI. Returns NULL if the URI isn't valid.
+ * Returns: The location for this URI. Returns NULL if the URI isn't valid. If
+ * the URI does not contain a location, an empty string is returned.
  */
 gchar *
 gst_uri_get_location (const gchar * uri)
 {
-  gchar *colon;
-  gchar *location, *unescaped;
+  const gchar *colon;
+  gchar *unescaped = NULL;
 
   g_return_val_if_fail (uri != NULL, NULL);
   g_return_val_if_fail (gst_uri_is_valid (uri), NULL);
 
   colon = strstr (uri, "://");
 
-  location = g_strdup (colon + 3);
+  unescaped = unescape_string (colon + 3, "/");
 
-  unescaped = unescape_string (location, "/");
-  g_free (location);
+  /* On Windows an URI might look like file:///c:/foo/bar.txt or
+   * file:///c|/foo/bar.txt (some Netscape versions) and we want to
+   * return c:/foo/bar.txt as location rather than /c:/foo/bar.txt.
+   * Can't use g_filename_from_uri() here because it will only handle the
+   * file:// protocol */
+#ifdef G_OS_WIN32
+  if (unescaped != NULL && unescaped[0] == '/' &&
+      g_ascii_isalpha (unescaped[1]) &&
+      (unescaped[2] == ':' || unescaped[2] == '|')) {
+    unescaped[2] = ':';
+    g_memmove (unescaped, unescaped + 1, strlen (unescaped + 1) + 1);
+  }
+#endif
 
+  GST_LOG ("extracted location '%s' from URI '%s'", GST_STR_NULL (unescaped),
+      uri);;
   return unescaped;
 }
 
@@ -456,7 +473,7 @@ gst_uri_construct (const gchar * protocol, const gchar * location)
 typedef struct
 {
   GstURIType type;
-  gchar *protocol;
+  const gchar *protocol;
 }
 SearchEntry;
 
@@ -475,10 +492,15 @@ search_by_entry (GstPluginFeature * feature, gpointer search_entry)
     return FALSE;
 
   protocols = gst_element_factory_get_uri_protocols (factory);
-  /* must be set when uri type is valid */
-  g_assert (protocols);
+
+  if (protocols == NULL) {
+    g_warning ("Factory '%s' implements GstUriHandler interface but returned "
+        "no supported protocols!", gst_plugin_feature_get_name (feature));
+    return FALSE;
+  }
+
   while (*protocols != NULL) {
-    if (strcmp (*protocols, entry->protocol) == 0)
+    if (g_ascii_strcasecmp (*protocols, entry->protocol) == 0)
       return TRUE;
     protocols++;
   }
@@ -495,9 +517,55 @@ sort_by_rank (gconstpointer a, gconstpointer b)
       gst_plugin_feature_get_rank (first);
 }
 
+static GList *
+get_element_factories_from_uri_protocol (const GstURIType type,
+    const gchar * protocol)
+{
+  GList *possibilities;
+  SearchEntry entry;
+
+  g_return_val_if_fail (protocol, NULL);
+
+  entry.type = type;
+  entry.protocol = protocol;
+  possibilities = gst_registry_feature_filter (gst_registry_get_default (),
+      search_by_entry, FALSE, &entry);
+
+  return possibilities;
+}
+
+/**
+ * gst_uri_protocol_is_supported:
+ * @type: Whether to check for a source or a sink
+ * @protocol: Protocol that should be checked for (e.g. "http" or "smb")
+ *
+ * Checks if an element exists that supports the given URI protocol. Note
+ * that a positive return value does not imply that a subsequent call to
+ * gst_element_make_from_uri() is guaranteed to work.
+ *
+ * Returns: TRUE
+ *
+ * Since: 0.10.13
+*/
+gboolean
+gst_uri_protocol_is_supported (const GstURIType type, const gchar * protocol)
+{
+  GList *possibilities;
+
+  g_return_val_if_fail (protocol, FALSE);
+
+  possibilities = get_element_factories_from_uri_protocol (type, protocol);
+
+  if (possibilities) {
+    g_list_free (possibilities);
+    return TRUE;
+  } else
+    return FALSE;
+}
+
 /**
  * gst_element_make_from_uri:
- * @type: Wether to create a source or a sink
+ * @type: Whether to create a source or a sink
  * @uri: URI to create an element for
  * @elementname: Name of created element, can be NULL.
  *
@@ -510,17 +578,15 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
     const gchar * elementname)
 {
   GList *possibilities, *walk;
-  SearchEntry entry;
+  gchar *protocol;
   GstElement *ret = NULL;
 
   g_return_val_if_fail (GST_URI_TYPE_IS_VALID (type), NULL);
   g_return_val_if_fail (gst_uri_is_valid (uri), NULL);
 
-  entry.type = type;
-  entry.protocol = gst_uri_get_protocol (uri);
-  possibilities = gst_registry_feature_filter (gst_registry_get_default (),
-      search_by_entry, FALSE, &entry);
-  g_free (entry.protocol);
+  protocol = gst_uri_get_protocol (uri);
+  possibilities = get_element_factories_from_uri_protocol (type, protocol);
+  g_free (protocol);
 
   if (!possibilities) {
     GST_DEBUG ("No %s for URI '%s'", type == GST_URI_SINK ? "sink" : "source",
@@ -542,7 +608,7 @@ gst_element_make_from_uri (const GstURIType type, const gchar * uri,
     }
     walk = walk->next;
   }
-  g_list_free (possibilities);
+  gst_plugin_feature_list_free (possibilities);
 
   GST_LOG_OBJECT (ret, "created %s for URL '%s'",
       type == GST_URI_SINK ? "sink" : "source", uri);
@@ -596,8 +662,14 @@ gst_uri_handler_get_protocols (GstURIHandler * handler)
 
   iface = GST_URI_HANDLER_GET_INTERFACE (handler);
   g_return_val_if_fail (iface != NULL, NULL);
-  g_return_val_if_fail (iface->get_protocols != NULL, NULL);
-  ret = iface->get_protocols ();
+  g_return_val_if_fail (iface->get_protocols != NULL ||
+      iface->get_protocols_full != NULL, NULL);
+
+  if (iface->get_protocols != NULL) {
+    ret = iface->get_protocols ();
+  } else {
+    ret = iface->get_protocols_full (G_OBJECT_TYPE (handler));
+  }
   g_return_val_if_fail (ret != NULL, NULL);
 
   return ret;
