@@ -230,11 +230,16 @@ struct _GstBaseTransformPrivate
   gboolean qos_enabled;
   gdouble proportion;
   GstClockTime earliest_time;
+  /* previous buffer had a discont */
+  gboolean discont;
+
+  GstActivateMode pad_mode;
+
+  gboolean gap_aware;
 };
 
 static GstElementClass *parent_class = NULL;
 
-static void gst_base_transform_base_init (gpointer g_class);
 static void gst_base_transform_class_init (GstBaseTransformClass * klass);
 static void gst_base_transform_init (GstBaseTransform * trans,
     GstBaseTransformClass * klass);
@@ -249,7 +254,7 @@ gst_base_transform_get_type (void)
   if (!base_transform_type) {
     static const GTypeInfo base_transform_info = {
       sizeof (GstBaseTransformClass),
-      (GBaseInitFunc) gst_base_transform_base_init,
+      NULL,
       NULL,
       (GClassInitFunc) gst_base_transform_class_init,
       NULL,
@@ -274,18 +279,18 @@ static gboolean gst_base_transform_src_activate_pull (GstPad * pad,
     gboolean active);
 static gboolean gst_base_transform_sink_activate_push (GstPad * pad,
     gboolean active);
+static gboolean gst_base_transform_activate (GstBaseTransform * trans,
+    gboolean active);
 static gboolean gst_base_transform_get_unit_size (GstBaseTransform * trans,
     GstCaps * caps, guint * size);
 
-static GstStateChangeReturn gst_base_transform_change_state (GstElement *
-    element, GstStateChange transition);
-
 static gboolean gst_base_transform_src_event (GstPad * pad, GstEvent * event);
 static gboolean gst_base_transform_src_eventfunc (GstBaseTransform * trans,
     GstEvent * event);
 static gboolean gst_base_transform_sink_event (GstPad * pad, GstEvent * event);
 static gboolean gst_base_transform_sink_eventfunc (GstBaseTransform * trans,
     GstEvent * event);
+static gboolean gst_base_transform_check_get_range (GstPad * pad);
 static GstFlowReturn gst_base_transform_getrange (GstPad * pad, guint64 offset,
     guint length, GstBuffer ** buffer);
 static GstFlowReturn gst_base_transform_chain (GstPad * pad,
@@ -297,13 +302,6 @@ static GstFlowReturn gst_base_transform_buffer_alloc (GstPad * pad,
 
 /* static guint gst_base_transform_signals[LAST_SIGNAL] = { 0 }; */
 
-static void
-gst_base_transform_base_init (gpointer g_class)
-{
-  GST_DEBUG_CATEGORY_INIT (gst_base_transform_debug, "basetransform", 0,
-      "basetransform element");
-}
-
 static void
 gst_base_transform_finalize (GObject * object)
 {
@@ -320,10 +318,11 @@ static void
 gst_base_transform_class_init (GstBaseTransformClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
-  gstelement_class = GST_ELEMENT_CLASS (klass);
+
+  GST_DEBUG_CATEGORY_INIT (gst_base_transform_debug, "basetransform", 0,
+      "basetransform element");
 
   g_type_class_add_private (klass, sizeof (GstBaseTransformPrivate));
 
@@ -335,14 +334,11 @@ gst_base_transform_class_init (GstBaseTransformClass * klass)
       GST_DEBUG_FUNCPTR (gst_base_transform_get_property);
 
   g_object_class_install_property (gobject_class, PROP_QOS,
-      g_param_spec_boolean ("qos", "QoS", "handle QoS messages",
+      g_param_spec_boolean ("qos", "QoS", "Handle Quality-of-Service events",
           DEFAULT_PROP_QOS, G_PARAM_READWRITE));
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_base_transform_finalize);
 
-  gstelement_class->change_state =
-      GST_DEBUG_FUNCPTR (gst_base_transform_change_state);
-
   klass->passthrough_on_same_caps = FALSE;
   klass->event = GST_DEBUG_FUNCPTR (gst_base_transform_sink_eventfunc);
   klass->src_event = GST_DEBUG_FUNCPTR (gst_base_transform_src_eventfunc);
@@ -386,6 +382,8 @@ gst_base_transform_init (GstBaseTransform * trans,
       GST_DEBUG_FUNCPTR (gst_base_transform_setcaps));
   gst_pad_set_event_function (trans->srcpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_src_event));
+  gst_pad_set_checkgetrange_function (trans->srcpad,
+      GST_DEBUG_FUNCPTR (gst_base_transform_check_get_range));
   gst_pad_set_getrange_function (trans->srcpad,
       GST_DEBUG_FUNCPTR (gst_base_transform_getrange));
   gst_pad_set_activatepull_function (trans->srcpad,
@@ -398,6 +396,8 @@ gst_base_transform_init (GstBaseTransform * trans,
   trans->priv->qos_enabled = DEFAULT_PROP_QOS;
   trans->cache_caps1 = NULL;
   trans->cache_caps2 = NULL;
+  trans->priv->pad_mode = GST_ACTIVATE_NONE;
+  trans->priv->gap_aware = FALSE;
 
   trans->passthrough = FALSE;
   if (bclass->transform == NULL) {
@@ -521,20 +521,21 @@ gst_base_transform_transform_size (GstBaseTransform * trans,
 no_in_size:
   {
     GST_DEBUG_OBJECT (trans, "could not get in_size");
-    g_warning ("could not get in_size");
+    g_warning ("%s: could not get in_size", GST_ELEMENT_NAME (trans));
     return FALSE;
   }
 no_multiple:
   {
     GST_DEBUG_OBJECT (trans, "Size %u is not a multiple of unit size %u", size,
         inunitsize);
-    g_warning ("Size %u is not a multiple of unit size %u", size, inunitsize);
+    g_warning ("%s: size %u is not a multiple of unit size %u",
+        GST_ELEMENT_NAME (trans), size, inunitsize);
     return FALSE;
   }
 no_out_size:
   {
     GST_DEBUG_OBJECT (trans, "could not get out_size");
-    g_warning ("could not get out_size");
+    g_warning ("%s: could not get out_size", GST_ELEMENT_NAME (trans));
     return FALSE;
   }
 }
@@ -811,6 +812,14 @@ gst_base_transform_setcaps (GstPad * pad, GstCaps * caps)
   /* we know this will work, we implement the setcaps */
   gst_pad_set_caps (otherpad, othercaps);
 
+  if (pad == trans->srcpad && trans->priv->pad_mode == GST_ACTIVATE_PULL) {
+    ret &= gst_pad_set_caps (otherpeer, othercaps);
+    if (!ret) {
+      GST_INFO_OBJECT (trans, "otherpeer setcaps(%" GST_PTR_FORMAT ") failed",
+          othercaps);
+    }
+  }
+
 done:
   if (otherpeer)
     gst_object_unref (otherpeer);
@@ -962,10 +971,14 @@ gst_base_transform_prepare_output_buffer (GstBaseTransform * trans,
     if (copy_inbuf && gst_buffer_is_writable (*out_buf))
       memcpy (GST_BUFFER_DATA (*out_buf), GST_BUFFER_DATA (in_buf), out_size);
 
-    gst_buffer_stamp (*out_buf, in_buf);
-    GST_BUFFER_FLAGS (*out_buf) |= GST_BUFFER_FLAGS (in_buf) &
-        (GST_BUFFER_FLAG_PREROLL | GST_BUFFER_FLAG_DISCONT |
-        GST_BUFFER_FLAG_IN_CAPS | GST_BUFFER_FLAG_DELTA_UNIT);
+    gst_buffer_copy_metadata (*out_buf, in_buf,
+        GST_BUFFER_COPY_FLAGS | GST_BUFFER_COPY_TIMESTAMPS);
+
+    /* Unset the GAP flag if the element is _not_ GAP aware. Otherwise
+     * it might create an output buffer that does not contain neutral data
+     * but still has the GAP flag on it! */
+    if (!trans->priv->gap_aware)
+      GST_BUFFER_FLAG_UNSET (*out_buf, GST_BUFFER_FLAG_GAP);
   }
 
 done:
@@ -1046,6 +1059,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   GstBaseTransform *trans;
   GstFlowReturn res;
   guint new_size;
+  gboolean issinkcaps = TRUE;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
 
@@ -1061,9 +1075,26 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   else
     GST_DEBUG_OBJECT (trans, "... and offset %" G_GUINT64_FORMAT, offset);
 
+  /* if have_same_caps was previously set to TRUE we need to double check if it
+   * hasn't changed */
+  if (trans->have_same_caps) {
+    GstCaps *sinkcaps;
+
+    GST_OBJECT_LOCK (trans->sinkpad);
+    sinkcaps = GST_PAD_CAPS (trans->sinkpad);
+    issinkcaps = sinkcaps && (gst_caps_is_equal (sinkcaps, caps));
+    GST_OBJECT_UNLOCK (trans->sinkpad);
+  }
+
   /* before any buffers are pushed, have_same_caps is TRUE; allocating can trigger
    * a renegotiation and change that to FALSE */
-  if (trans->have_same_caps) {
+
+  /* bilboed: This seems wrong, from all debug logs, have_same_caps is
+   * initialized to FALSE */
+
+  /* checking against trans->have_same_caps is not enough !! It should also
+   *  check to see if the requested caps are equal to the sink caps */
+  if (trans->have_same_caps && issinkcaps) {
     /* request a buffer with the same caps */
     GST_DEBUG_OBJECT (trans, "requesting buffer with same caps, size %d", size);
 
@@ -1163,7 +1194,7 @@ not_configured:
     /* let the default allocator handle it... */
     GST_DEBUG_OBJECT (trans, "not configured");
     gst_buffer_replace (buf, NULL);
-    if (trans->passthrough || trans->always_in_place) {
+    if (trans->passthrough) {
       /* ...by calling alloc_buffer without setting caps on the src pad, which
        * will force negotiation in the chain function. */
       res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
@@ -1178,7 +1209,7 @@ unknown_size:
     /* let the default allocator handle it... */
     GST_DEBUG_OBJECT (trans, "unknown size");
     gst_buffer_replace (buf, NULL);
-    if (trans->passthrough || trans->always_in_place) {
+    if (trans->passthrough) {
       /* ...by calling alloc_buffer without setting caps on the src pad, which
        * will force negotiation in the chain function. */
       res = gst_pad_alloc_buffer (trans->srcpad, offset, size, caps, buf);
@@ -1196,16 +1227,17 @@ gst_base_transform_sink_event (GstPad * pad, GstEvent * event)
   GstBaseTransform *trans;
   GstBaseTransformClass *bclass;
   gboolean ret = TRUE;
+  gboolean forward = TRUE;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
   bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
 
   if (bclass->event)
-    ret = bclass->event (trans, event);
+    forward = bclass->event (trans, event);
 
   /* FIXME, do this in the default event handler so the subclass can do
    * something different. */
-  if (ret)
+  if (forward)
     ret = gst_pad_push_event (trans->srcpad, event);
 
   gst_object_unref (trans);
@@ -1224,6 +1256,7 @@ gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
       /* reset QoS parameters */
       trans->priv->proportion = 1.0;
       trans->priv->earliest_time = -1;
+      trans->priv->discont = FALSE;
       GST_OBJECT_UNLOCK (trans);
       /* we need new segment info after the flush. */
       trans->have_newsegment = FALSE;
@@ -1245,6 +1278,9 @@ gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
 
       trans->have_newsegment = TRUE;
 
+      gst_segment_set_newsegment_full (&trans->segment, update, rate, arate,
+          format, start, stop, time);
+
       if (format == GST_FORMAT_TIME) {
         GST_DEBUG_OBJECT (trans, "received TIME NEW_SEGMENT %" GST_TIME_FORMAT
             " -- %" GST_TIME_FORMAT ", time %" GST_TIME_FORMAT
@@ -1260,9 +1296,6 @@ gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
             trans->segment.start, trans->segment.stop,
             trans->segment.time, trans->segment.accum);
       }
-
-      gst_segment_set_newsegment_full (&trans->segment, update, rate, arate,
-          format, start, stop, time);
       break;
     }
     default:
@@ -1346,6 +1379,12 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
   if (!trans->negotiated && !trans->passthrough && (bclass->set_caps != NULL))
     goto not_negotiated;
 
+  /* Set discont flag so we can mark the outgoing buffer */
+  if (GST_BUFFER_IS_DISCONT (inbuf)) {
+    GST_LOG_OBJECT (trans, "got DISCONT buffer %p", inbuf);
+    trans->priv->discont = TRUE;
+  }
+
   /* can only do QoS if the segment is in TIME */
   if (trans->segment.format != GST_FORMAT_TIME)
     goto no_qos;
@@ -1369,6 +1408,8 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
       GST_CAT_DEBUG_OBJECT (GST_CAT_QOS, trans, "skipping transform: qostime %"
           GST_TIME_FORMAT " <= %" GST_TIME_FORMAT,
           GST_TIME_ARGS (qostime), GST_TIME_ARGS (earliest_time));
+      /* mark discont for next buffer */
+      trans->priv->discont = TRUE;
       goto skip;
     }
   }
@@ -1385,7 +1426,7 @@ no_qos:
 
     *outbuf = inbuf;
 
-    return ret;
+    goto done;
   }
 
   want_in_place = (bclass->transform_ip != NULL) && trans->always_in_place;
@@ -1451,6 +1492,7 @@ skip:
   if (*outbuf != inbuf)
     gst_buffer_unref (inbuf);
 
+done:
   return ret;
 
   /* ERRORS */
@@ -1485,6 +1527,21 @@ configure_failed:
   }
 }
 
+static gboolean
+gst_base_transform_check_get_range (GstPad * pad)
+{
+  GstBaseTransform *trans;
+  gboolean ret;
+
+  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+
+  ret = gst_pad_check_pull_range (trans->sinkpad);
+
+  gst_object_unref (trans);
+
+  return ret;
+}
+
 /* FIXME, getrange is broken, need to pull range from the other
  * end based on the transform_size result.
  */
@@ -1515,24 +1572,51 @@ gst_base_transform_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstBaseTransform *trans;
   GstFlowReturn ret;
+  GstClockTime last_stop = GST_CLOCK_TIME_NONE;
   GstBuffer *outbuf = NULL;
 
-  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  trans = GST_BASE_TRANSFORM (GST_OBJECT_PARENT (pad));
+
+  /* calculate end position of the incoming buffer */
+  if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE) {
+    if (GST_BUFFER_DURATION (buffer) != GST_CLOCK_TIME_NONE)
+      last_stop = GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer);
+    else
+      last_stop = GST_BUFFER_TIMESTAMP (buffer);
+  }
 
   /* protect transform method and concurrent buffer alloc */
   g_mutex_lock (trans->transform_lock);
   ret = gst_base_transform_handle_buffer (trans, buffer, &outbuf);
   g_mutex_unlock (trans->transform_lock);
 
-  /* outbuf can be NULL, this means a dropped buffer */
+  /* outbuf can be NULL, this means a dropped buffer, if we have a buffer but
+   * GST_BASE_TRANSFORM_FLOW_DROPPED we will not push either. */
   if (outbuf != NULL) {
-    if (ret == GST_FLOW_OK)
+    if ((ret == GST_FLOW_OK)) {
+      /* Remember last stop position */
+      if ((last_stop != GST_CLOCK_TIME_NONE) &&
+          (trans->segment.format == GST_FORMAT_TIME))
+        gst_segment_set_last_stop (&trans->segment, GST_FORMAT_TIME, last_stop);
+
+      /* apply DISCONT flag if the buffer is not yet marked as such */
+      if (trans->priv->discont) {
+        if (!GST_BUFFER_IS_DISCONT (outbuf)) {
+          outbuf = gst_buffer_make_metadata_writable (outbuf);
+          GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+        }
+        trans->priv->discont = FALSE;
+      }
       ret = gst_pad_push (trans->srcpad, outbuf);
-    else
+    } else
       gst_buffer_unref (outbuf);
   }
 
-  gst_object_unref (trans);
+  /* convert internal flow to OK and mark discont for the next buffer. */
+  if (ret == GST_BASE_TRANSFORM_FLOW_DROPPED) {
+    trans->priv->discont = TRUE;
+    ret = GST_FLOW_OK;
+  }
 
   return ret;
 }
@@ -1573,20 +1657,71 @@ gst_base_transform_get_property (GObject * object, guint prop_id,
   }
 }
 
+/* not a vmethod of anything, just an internal method */
+static gboolean
+gst_base_transform_activate (GstBaseTransform * trans, gboolean active)
+{
+  GstBaseTransformClass *bclass;
+  gboolean result = TRUE;
+
+  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
+
+  if (active) {
+    if (trans->priv->pad_mode == GST_ACTIVATE_NONE && bclass->start)
+      result &= bclass->start (trans);
+
+    GST_OBJECT_LOCK (trans);
+
+    if (GST_PAD_CAPS (trans->sinkpad) && GST_PAD_CAPS (trans->srcpad))
+      trans->have_same_caps =
+          gst_caps_is_equal (GST_PAD_CAPS (trans->sinkpad),
+          GST_PAD_CAPS (trans->srcpad)) || trans->passthrough;
+    else
+      trans->have_same_caps = trans->passthrough;
+    GST_DEBUG_OBJECT (trans, "have_same_caps %d", trans->have_same_caps);
+    trans->negotiated = FALSE;
+    trans->have_newsegment = FALSE;
+    gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
+    trans->priv->proportion = 1.0;
+    trans->priv->earliest_time = -1;
+    trans->priv->discont = FALSE;
+
+    GST_OBJECT_UNLOCK (trans);
+  } else {
+    /* We must make sure streaming has finished before resetting things
+     * and calling the ::stop vfunc */
+    GST_PAD_STREAM_LOCK (trans->sinkpad);
+    GST_PAD_STREAM_UNLOCK (trans->sinkpad);
+
+    trans->have_same_caps = FALSE;
+    /* We can only reset the passthrough mode if the instance told us to 
+       handle it in configure_caps */
+    if (bclass->passthrough_on_same_caps) {
+      gst_base_transform_set_passthrough (trans, FALSE);
+    }
+    gst_caps_replace (&trans->cache_caps1, NULL);
+    gst_caps_replace (&trans->cache_caps2, NULL);
+
+    if (trans->priv->pad_mode != GST_ACTIVATE_NONE && bclass->stop)
+      result &= bclass->stop (trans);
+  }
+
+  return result;
+}
+
 static gboolean
 gst_base_transform_sink_activate_push (GstPad * pad, gboolean active)
 {
   gboolean result = TRUE;
   GstBaseTransform *trans;
-  GstBaseTransformClass *bclass;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
-  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
 
-  if (active) {
-    if (bclass->start)
-      result = bclass->start (trans);
-  }
+  result = gst_base_transform_activate (trans, active);
+
+  if (result)
+    trans->priv->pad_mode = active ? GST_ACTIVATE_PUSH : GST_ACTIVATE_NONE;
+
   gst_object_unref (trans);
 
   return result;
@@ -1597,85 +1732,22 @@ gst_base_transform_src_activate_pull (GstPad * pad, gboolean active)
 {
   gboolean result = FALSE;
   GstBaseTransform *trans;
-  GstBaseTransformClass *bclass;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
-  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
 
   result = gst_pad_activate_pull (trans->sinkpad, active);
 
-  if (active) {
-    if (result && bclass->start)
-      result &= bclass->start (trans);
-  }
+  if (result)
+    result &= gst_base_transform_activate (trans, active);
+
+  if (result)
+    trans->priv->pad_mode = active ? GST_ACTIVATE_PULL : GST_ACTIVATE_NONE;
+
   gst_object_unref (trans);
 
   return result;
 }
 
-static GstStateChangeReturn
-gst_base_transform_change_state (GstElement * element,
-    GstStateChange transition)
-{
-  GstBaseTransform *trans;
-  GstBaseTransformClass *bclass;
-  GstStateChangeReturn result;
-
-  trans = GST_BASE_TRANSFORM (element);
-  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
-
-
-  switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      GST_OBJECT_LOCK (trans);
-      if (GST_PAD_CAPS (trans->sinkpad) && GST_PAD_CAPS (trans->srcpad))
-        trans->have_same_caps =
-            gst_caps_is_equal (GST_PAD_CAPS (trans->sinkpad),
-            GST_PAD_CAPS (trans->srcpad)) || trans->passthrough;
-      else
-        trans->have_same_caps = trans->passthrough;
-      GST_DEBUG_OBJECT (trans, "have_same_caps %d", trans->have_same_caps);
-      trans->negotiated = FALSE;
-      trans->have_newsegment = FALSE;
-      gst_segment_init (&trans->segment, GST_FORMAT_UNDEFINED);
-      trans->priv->proportion = 1.0;
-      trans->priv->earliest_time = -1;
-      GST_OBJECT_UNLOCK (trans);
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      break;
-    default:
-      break;
-  }
-
-  result = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      trans->have_same_caps = FALSE;
-      /* We can only reset the passthrough mode if the instance told us to 
-         handle it in configure_caps */
-      if (bclass->passthrough_on_same_caps) {
-        gst_base_transform_set_passthrough (trans, FALSE);
-      }
-      gst_caps_replace (&trans->cache_caps1, NULL);
-      gst_caps_replace (&trans->cache_caps2, NULL);
-      if (bclass->stop)
-        result = bclass->stop (trans);
-      break;
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      break;
-    default:
-      break;
-  }
-
-  return result;
-}
-
 /**
  * gst_base_transform_set_passthrough:
  * @trans: the #GstBaseTransform to set
@@ -1878,3 +1950,29 @@ gst_base_transform_is_qos_enabled (GstBaseTransform * trans)
 
   return result;
 }
+
+/**
+ * gst_base_transform_set_gap_aware:
+ * @trans: a #GstBaseTransform
+ * @gap_aware: New state
+ *
+ * If @gap_aware is %FALSE (as it is by default) subclasses will never get
+ * output buffers with the %GST_BUFFER_FLAG_GAP flag set.
+ *
+ * If set to %TRUE elements must handle output buffers with this flag set
+ * correctly, i.e. they can assume that the buffer contains neutral data
+ * but must unset the flag if the output is no neutral data.
+ * Since: 0.10.16
+ *
+ * MT safe.
+ */
+void
+gst_base_transform_set_gap_aware (GstBaseTransform * trans, gboolean gap_aware)
+{
+  g_return_if_fail (trans != NULL);
+
+  GST_OBJECT_LOCK (trans);
+  trans->priv->gap_aware = gap_aware;
+  GST_DEBUG_OBJECT (trans, "set gap aware %d", trans->priv->gap_aware);
+  GST_OBJECT_UNLOCK (trans);
+}
