@@ -118,12 +118,6 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (gst_file_src_debug);
 #define GST_CAT_DEFAULT gst_file_src_debug
 
-static const GstElementDetails gst_file_src_details =
-GST_ELEMENT_DETAILS ("File Source",
-    "Source/File",
-    "Read from arbitrary point in a file",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
 /* FileSrc signals and args */
 enum
 {
@@ -188,10 +182,13 @@ gst_file_src_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
+  gst_element_class_set_details_simple (gstelement_class,
+      "File Source",
+      "Source/File",
+      "Read from arbitrary point in a file",
+      "Erik Walthinsen <omega@cse.ogi.edu>");
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&srctemplate));
-
-  gst_element_class_set_details (gstelement_class, &gst_file_src_details);
 }
 
 static void
@@ -260,7 +257,8 @@ gst_file_src_class_init (GstFileSrcClass * klass)
   gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_file_src_create);
 
   if (sizeof (off_t) < 8) {
-    GST_LOG ("No large file support, sizeof (off_t) = %u!", sizeof (off_t));
+    GST_LOG ("No large file support, sizeof (off_t) = %" G_GSIZE_FORMAT "!",
+        sizeof (off_t));
   }
 }
 
@@ -301,16 +299,14 @@ gst_file_src_finalize (GObject * object)
 static gboolean
 gst_file_src_set_location (GstFileSrc * src, const gchar * location)
 {
-  /* the element must be stopped in order to do this */
-  GST_STATE_LOCK (src);
-  {
-    GstState state;
+  GstState state;
 
-    state = GST_STATE (src);
-    if (state != GST_STATE_READY && state != GST_STATE_NULL)
-      goto wrong_state;
-  }
-  GST_STATE_UNLOCK (src);
+  /* the element must be stopped in order to do this */
+  GST_OBJECT_LOCK (src);
+  state = GST_STATE (src);
+  if (state != GST_STATE_READY && state != GST_STATE_NULL)
+    goto wrong_state;
+  GST_OBJECT_UNLOCK (src);
 
   g_free (src->filename);
   g_free (src->uri);
@@ -332,7 +328,7 @@ gst_file_src_set_location (GstFileSrc * src, const gchar * location)
 wrong_state:
   {
     GST_DEBUG_OBJECT (src, "setting location in wrong state");
-    GST_STATE_UNLOCK (src);
+    GST_OBJECT_UNLOCK (src);
     return FALSE;
   }
 }
@@ -446,6 +442,7 @@ struct _GstMmapBufferClass
 static void gst_mmap_buffer_init (GTypeInstance * instance, gpointer g_class);
 static void gst_mmap_buffer_class_init (gpointer g_class, gpointer class_data);
 static void gst_mmap_buffer_finalize (GstMmapBuffer * mmap_buffer);
+static GstBufferClass *mmap_buffer_parent_class = NULL;
 
 static GType
 gst_mmap_buffer_get_type (void)
@@ -477,6 +474,8 @@ gst_mmap_buffer_class_init (gpointer g_class, gpointer class_data)
 {
   GstMiniObjectClass *mini_object_class = GST_MINI_OBJECT_CLASS (g_class);
 
+  mmap_buffer_parent_class = g_type_class_peek_parent (g_class);
+
   mini_object_class->finalize =
       (GstMiniObjectFinalizeFunction) gst_mmap_buffer_finalize;
 }
@@ -512,23 +511,26 @@ gst_mmap_buffer_finalize (GstMmapBuffer * mmap_buffer)
 #ifdef MADV_DONTNEED
   /* madvise to tell the kernel what to do with it */
   if (madvise (data, size, MADV_DONTNEED) < 0) {
-    GST_WARNING_OBJECT (src, "warning: madvise failed: %s", strerror (errno));
+    GST_WARNING_OBJECT (src, "warning: madvise failed: %s", g_strerror (errno));
   }
 #endif
 
   /* now unmap the memory */
   if (munmap (data, size) < 0) {
-    GST_WARNING_OBJECT (src, "warning: munmap failed: %s", strerror (errno));
+    GST_WARNING_OBJECT (src, "warning: munmap failed: %s", g_strerror (errno));
   }
 
   /* cast to unsigned long, since there's no gportable way to print
    * guint64 as hex */
   GST_LOG ("unmapped region %08lx+%08lx at %p",
       (gulong) offset, (gulong) size, data);
+
+  GST_MINI_OBJECT_CLASS (mmap_buffer_parent_class)->
+      finalize (GST_MINI_OBJECT (mmap_buffer));
 }
 
 static GstBuffer *
-gst_file_src_map_region (GstFileSrc * src, off_t offset, size_t size,
+gst_file_src_map_region (GstFileSrc * src, off_t offset, gsize size,
     gboolean testonly)
 {
   GstBuffer *buf;
@@ -557,7 +559,8 @@ gst_file_src_map_region (GstFileSrc * src, off_t offset, size_t size,
   if (src->sequential) {
     /* madvise to tell the kernel what to do with it */
     if (madvise (mmapregion, size, MADV_SEQUENTIAL) < 0) {
-      GST_WARNING_OBJECT (src, "warning: madvise failed: %s", strerror (errno));
+      GST_WARNING_OBJECT (src, "warning: madvise failed: %s",
+          g_strerror (errno));
     }
   }
 #endif
@@ -574,24 +577,24 @@ gst_file_src_map_region (GstFileSrc * src, off_t offset, size_t size,
 mmap_failed:
   {
     if (!testonly) {
-      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-          ("mmap (0x%08lx, %d, 0x%llx) failed: %s",
-              (gulong) size, src->fd, offset, strerror (errno)));
+      GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
+          ("mmap (0x%08lx, %d, 0x%" G_GINT64_MODIFIER "x) failed: %s",
+              (gulong) size, src->fd, (guint64) offset, g_strerror (errno)));
     }
     return NULL;
   }
 }
 
 static GstBuffer *
-gst_file_src_map_small_region (GstFileSrc * src, off_t offset, size_t size)
+gst_file_src_map_small_region (GstFileSrc * src, off_t offset, gsize size)
 {
   GstBuffer *ret;
   off_t mod;
   guint pagesize;
 
   GST_LOG_OBJECT (src,
-      "attempting to map a small buffer at %llu+%d",
-      (unsigned long long) offset, (gint) size);
+      "attempting to map a small buffer at %" G_GUINT64_FORMAT "+%d",
+      (guint64) offset, (gint) size);
 
   pagesize = src->pagesize;
 
@@ -599,7 +602,7 @@ gst_file_src_map_small_region (GstFileSrc * src, off_t offset, size_t size)
 
   /* if the offset starts at a non-page boundary, we have to special case */
   if (mod != 0) {
-    size_t mapsize;
+    gsize mapsize;
     off_t mapbase;
     GstBuffer *map;
 
@@ -607,8 +610,8 @@ gst_file_src_map_small_region (GstFileSrc * src, off_t offset, size_t size)
     mapsize = ((size + mod + pagesize - 1) / pagesize) * pagesize;
 
     GST_LOG_OBJECT (src,
-        "not on page boundaries, resizing to map to %llu+%d",
-        (unsigned long long) mapbase, (gint) mapsize);
+        "not on page boundaries, resizing to map to %" G_GUINT64_FORMAT "+%d",
+        (guint64) mapbase, (gint) mapsize);
 
     map = gst_file_src_map_region (src, mapbase, mapsize, FALSE);
     if (map == NULL)
@@ -630,7 +633,7 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
     GstBuffer ** buffer)
 {
   GstBuffer *buf = NULL;
-  size_t readsize, mapsize;
+  gsize readsize, mapsize;
   off_t readend, mapstart, mapend;
   int i;
 
@@ -651,18 +654,17 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
     /* if the end is before the mapend, the buffer is in current mmap region... */
     /* ('cause by definition if readend is in the buffer, so's readstart) */
     if (readend <= mapend) {
-      GST_LOG_OBJECT (src,
-          "read buf %llu+%d lives in current mapbuf %lld+%d, creating subbuffer of mapbuf",
-          offset, (int) readsize, mapstart, mapsize);
+      GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%u lives in "
+          "current mapbuf %u+%u, creating subbuffer of mapbuf",
+          offset, (guint) readsize, (guint) mapstart, (guint) mapsize);
       buf = gst_buffer_create_sub (src->mapbuf, offset - mapstart, readsize);
       GST_BUFFER_OFFSET (buf) = offset;
 
       /* if the start actually is within the current mmap region, map an overlap buffer */
     } else if (offset < mapend) {
-      GST_LOG_OBJECT (src,
-          "read buf %llu+%d starts in mapbuf %d+%d but ends outside, creating new mmap",
-          (unsigned long long) offset, (gint) readsize, (gint) mapstart,
-          (gint) mapsize);
+      GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%u starts in "
+          "mapbuf %u+%u but ends outside, creating new mmap",
+          offset, (guint) readsize, (guint) mapstart, (guint) mapsize);
       buf = gst_file_src_map_small_region (src, offset, readsize);
       if (buf == NULL)
         goto could_not_mmap;
@@ -676,10 +678,9 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
     /* either the read buffer overlaps the start of the mmap region */
     /* or the read buffer fully contains the current mmap region    */
     /* either way, it's really not relevant, we just create a new region anyway */
-    GST_LOG_OBJECT (src,
-        "read buf %llu+%d starts before mapbuf %d+%d, but overlaps it",
-        (unsigned long long) offset, (gint) readsize, (gint) mapstart,
-        (gint) mapsize);
+    GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%d starts before "
+        "mapbuf %d+%d, but overlaps it", (guint64) offset, (gint) readsize,
+        (gint) mapstart, (gint) mapsize);
     buf = gst_file_src_map_small_region (src, offset, readsize);
     if (buf == NULL)
       goto could_not_mmap;
@@ -688,27 +689,27 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
   /* then deal with the case where the read buffer is totally outside */
   if (buf == NULL) {
     /* first check to see if there's a map that covers the right region already */
-    GST_LOG_OBJECT (src, "searching for mapbuf to cover %llu+%d",
-        offset, (int) readsize);
+    GST_LOG_OBJECT (src, "searching for mapbuf to cover %" G_GUINT64_FORMAT
+        "+%d", offset, (int) readsize);
 
     /* if the read buffer crosses a mmap region boundary, create a one-off region */
     if ((offset / src->mapsize) != (readend / src->mapsize)) {
-      GST_LOG_OBJECT (src,
-          "read buf %llu+%d crosses a %d-byte boundary, creating a one-off",
-          offset, (int) readsize, (int) src->mapsize);
+      GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%d crosses a "
+          "%d-byte boundary, creating a one-off", offset, (int) readsize,
+          (int) src->mapsize);
       buf = gst_file_src_map_small_region (src, offset, readsize);
       if (buf == NULL)
         goto could_not_mmap;
 
       /* otherwise we will create a new mmap region and set it to the default */
     } else {
-      size_t mapsize;
+      gsize mapsize;
 
       off_t nextmap = offset - (offset % src->mapsize);
 
-      GST_LOG_OBJECT (src,
-          "read buf %llu+%d in new mapbuf at %llu+%d, mapping and subbuffering",
-          offset, readsize, nextmap, src->mapsize);
+      GST_LOG_OBJECT (src, "read buf %" G_GUINT64_FORMAT "+%d in new mapbuf "
+          "at %" G_GUINT64_FORMAT "+%d, mapping and subbuffering",
+          offset, (gint) readsize, (guint64) nextmap, (gint) src->mapsize);
       /* first, we're done with the old mapbuf */
       gst_buffer_unref (src->mapbuf);
       mapsize = src->mapsize;
@@ -716,7 +717,7 @@ gst_file_src_create_mmap (GstFileSrc * src, guint64 offset, guint length,
       /* double the mapsize as long as the readsize is smaller */
       while (readsize + offset > nextmap + mapsize) {
         GST_LOG_OBJECT (src, "readsize smaller then mapsize %08x %d",
-            readsize, (int) mapsize);
+            (guint) readsize, (gint) mapsize);
         mapsize <<= 1;
       }
       /* create a new one */
@@ -788,8 +789,8 @@ gst_file_src_create_read (GstFileSrc * src, guint64 offset, guint length,
   if (G_UNLIKELY (ret < 0))
     goto could_not_read;
 
-  /* regular files should have given us what we expected */
-  if (G_UNLIKELY ((guint) ret < length && src->is_regular))
+  /* seekable regular files should have given us what we expected */
+  if (G_UNLIKELY ((guint) ret < length && src->seekable))
     goto unexpected_eos;
 
   /* other files should eos if they read 0 and more was requested */
@@ -943,15 +944,16 @@ gst_file_src_start (GstBaseSrc * basesrc)
   {
     /* If not in mmap mode, we need to check if the underlying file is
      * seekable. */
-    off_t res = lseek (src->fd, 0, SEEK_CUR);
+    off_t res = lseek (src->fd, 0, SEEK_END);
 
     if (res < 0) {
       GST_LOG_OBJECT (src, "disabling seeking, not in mmap mode and lseek "
-          "failed: %s", strerror (errno));
+          "failed: %s", g_strerror (errno));
       src->seekable = FALSE;
     } else {
       src->seekable = TRUE;
     }
+    lseek (src->fd, 0, SEEK_SET);
   }
 
   /* We can only really do seeking on regular files - for other file types, we
@@ -976,8 +978,8 @@ open_failed:
         break;
       default:
         GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-            (_("Could not open file \"%s\" for reading: %s."), src->filename,
-                strerror (errno)), GST_ERROR_SYSTEM);
+            (_("Could not open file \"%s\" for reading."), src->filename),
+            GST_ERROR_SYSTEM);
         break;
     }
     return FALSE;
@@ -1028,7 +1030,7 @@ gst_file_src_stop (GstBaseSrc * basesrc)
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
-static guint
+static GstURIType
 gst_file_src_uri_get_type (void)
 {
   return GST_URI_SRC;
@@ -1061,7 +1063,35 @@ gst_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri)
     return FALSE;
   }
   g_free (protocol);
-  location = gst_uri_get_location (uri);
+
+  /* allow file://localhost/foo/bar by stripping localhost but fail
+   * for every other hostname */
+  if (g_str_has_prefix (uri, "file://localhost/")) {
+    char *tmp;
+
+    /* 16 == strlen ("file://localhost") */
+    tmp = g_strconcat ("file://", uri + 16, NULL);
+    /* we use gst_uri_get_location() although we already have the
+     * "location" with uri + 16 because it provides unescaping */
+    location = gst_uri_get_location (tmp);
+    g_free (tmp);
+  } else if (strcmp (uri, "file://") == 0) {
+    /* Special case for "file://" as this is used by some applications
+     *  to test with gst_element_make_from_uri if there's an element
+     *  that supports the URI protocol. */
+    gst_file_src_set_location (src, NULL);
+    return TRUE;
+  } else {
+    location = gst_uri_get_location (uri);
+  }
+
+  if (!location)
+    return FALSE;
+  if (!g_path_is_absolute (location)) {
+    g_free (location);
+    return FALSE;
+  }
+
   ret = gst_file_src_set_location (src, location);
   g_free (location);
 
